# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

from __future__ import annotations

from importlib.metadata import version as _version

VERSION: str = _version("license_tools")

del _version



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

"""
Tools related to binary linking.
"""

from __future__ import annotations

import logging
import os
import shutil
import subprocess
from pathlib import Path
from typing import Literal, cast

from license_tools.utils.path_utils import get_file_type

logger = logging.getLogger(__name__)
del logging


ELF_EXE = "executable"
ELF_SHARED = "shared object"
ELF_RELOC = "relocatable"
ELF_UNKNOWN = "unknown"
ELF_TYPES = [ELF_EXE, ELF_SHARED, ELF_RELOC]
ELF_TYPES_TYPE = Literal["executable", "shared object", "relocatable", "unknown"]


def is_elf(path: Path) -> bool:
    """
    Check if the given file is an ELF file.

    :param path: The file to check.
    :return: True if the file is an ELF binary, False otherwise.
    """
    file_type = get_file_type(path).lower()
    return file_type.startswith("elf") and any(elf_type in file_type for elf_type in ELF_TYPES)


def get_elf_type(path: Path) -> ELF_TYPES_TYPE | None:
    """
    Get the ELF type of the given file.

    :param path: The file to check.
    :return: The ELF type of the given file if it is an ELF binary, None otherwise.
    """
    if not is_elf(path):
        return None
    file_type = get_file_type(path).lower()
    for elf_type in ELF_TYPES:
        if elf_type in file_type:
            return cast(ELF_TYPES_TYPE, elf_type)
    return cast(ELF_TYPES_TYPE, ELF_UNKNOWN)


def check_shared_objects(path: Path) -> str | None:
    """
    Check which other shared objects a shared object links to.

    :param path: The file path to analyze.
    :return: The analysis results if the path points to a shared object, `None` otherwise.
    """
    if not is_elf(path):
        return None
    if path.is_symlink():
        # Ignore symlinks as they usually are package-internal and `ldd` does not always like them.
        logger.warning(
            "Ignoring symlink %s to %s for shared object analysis.", path, path.resolve(),
        )
        return None

    ldd = shutil.which("ldd")
    if not ldd:
        raise ValueError("ldd not found!")

    env = os.environ.copy()
    env["LC_ALL"] = "C"

    try:
        output = subprocess.check_output([ldd, path], stderr=subprocess.PIPE, env=env)
    except subprocess.CalledProcessError as exception:
        if b"\tnot a dynamic executable" in exception.stderr:
            return None
        raise
    return output.decode("UTF-8")



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

from __future__ import annotations

import re
from pathlib import Path
from tempfile import TemporaryDirectory
from unittest import TestCase, mock

from license_tools.tools import cargo_tools
from license_tools.tools.cargo_tools import PackageVersion
from license_tools.utils.download_utils import Download
from tests import get_from_url
from tests.data import BASE64__0_22_0__CARGO_TOML, CRYPTOGRAPHY__42_0_0__CARGO_LOCK

EXPECTED_METADATA = {
    "authors": ["Alice Maz <alice@alicemaz.com>", "Marshall Pierce <marshall@mpierce.org>"],
    "categories": ["encoding"],
    "description": "encodes and decodes base64 as bytes or utf8",
    "documentation": "https://docs.rs/base64",
    "edition": "2018",
    "keywords": ["base64", "utf8", "encode", "decode", "no_std"],
    "license": "MIT OR Apache-2.0",
    "metadata": {"docs": {"rs": {"rustdoc-args": ["--generate-link-to-definition"]}}},
    "name": "base64",
    "readme": "README.md",
    "repository": "https://github.com/marshallpierce/rust-base64",
    "rust-version": "1.48.0",
    "version": "0.22.0",
}


class ReadTomlTestCase(TestCase):
    def test_read_toml(self) -> None:
        with get_from_url(BASE64__0_22_0__CARGO_TOML) as path:
            result = cargo_tools.read_toml(path)
        self.assertEqual(
            {
                "bench": [{"harness": False, "name": "benchmarks", "required-features": ["std"]}],
                "dev-dependencies": {
                    "clap": {"features": ["derive"], "version": "3.2.25"},
                    "criterion": "0.4.0",
                    "once_cell": "1",
                    "rand": {"features": ["small_rng"], "version": "0.8.5"},
                    "rstest": "0.13.0",
                    "rstest_reuse": "0.6.0",
                    "strum": {"features": ["derive"], "version": "0.25"},
                },
                "example": [{"name": "base64", "required-features": ["std"]}],
                "features": {"alloc": [], "default": ["std"], "std": ["alloc"]},
                "package": EXPECTED_METADATA,
                "profile": {"bench": {"debug": True}, "test": {"opt-level": 3}},
                "test": [{"name": "tests", "required-features": ["alloc"]}, {"name": "encode", "required-features": ["alloc"]}],
            },
            result,
        )


class AnalyzeMetadataTestCase(TestCase):
    def test_path_is_cargo_toml(self) -> None:
        with get_from_url(BASE64__0_22_0__CARGO_TOML) as path, TemporaryDirectory() as directory:
            cargo_toml = Path(directory) / "Cargo.toml"
            cargo_toml.write_bytes(path.read_bytes())
            metadata = cargo_tools.analyze_metadata(cargo_toml)
        self.assertEqual(EXPECTED_METADATA, metadata)

    def test_path_is_parent_of_cargo_toml(self) -> None:
        with get_from_url(BASE64__0_22_0__CARGO_TOML) as path, TemporaryDirectory() as directory:
            cargo_toml = Path(directory) / "Cargo.toml"
            cargo_toml.write_bytes(path.read_bytes())
            metadata = cargo_tools.analyze_metadata(Path(directory))
        self.assertEqual(EXPECTED_METADATA, metadata)

    def test_path_is_grandparent_of_cargo_toml(self) -> None:
        with get_from_url(BASE64__0_22_0__CARGO_TOML) as path, TemporaryDirectory() as directory:
            cargo_toml = Path(directory) / "base64-0.22.1" / "Cargo.toml"
            cargo_toml.parent.mkdir()
            cargo_toml.write_bytes(path.read_bytes())
            metadata = cargo_tools.analyze_metadata(Path(directory))
            self.assertEqual(EXPECTED_METADATA, metadata)

            Path(directory, "another_directory").mkdir()
            with self.assertRaisesRegex(expected_exception=ValueError, expected_regex=rf"^No clear Cargo\.toml in {re.escape(directory)}\.$"):
                cargo_tools.analyze_metadata(Path(directory))


class CheckMetadataTestCase(TestCase):
    def test_check_metadata(self) -> None:
        with get_from_url(BASE64__0_22_0__CARGO_TOML) as path, TemporaryDirectory() as directory:
            cargo_toml = Path(directory) / "Cargo.toml"
            cargo_toml.write_bytes(path.read_bytes())
            metadata = cargo_tools.check_metadata(cargo_toml)
        self.assertEqual(
            """
        Name: base64
     Version: 0.22.0
     Authors:
               * Alice Maz <alice@alicemaz.com>
               * Marshall Pierce <marshall@mpierce.org>
 Description: encodes and decodes base64 as bytes or utf8
      README: README.md
  Repository: https://github.com/marshallpierce/rust-base64
     License: MIT OR Apache-2.0
    Keywords:
               * base64
               * decode
               * encode
               * no_std
               * utf8
  Categories: encoding
"""[
                1:-1
            ],
            metadata,
        )


class PackageVersionTestCase(TestCase):
    def test_to_download(self) -> None:
        package_version = PackageVersion(name="autocfg", version="1.1.0", checksum="d468802bab17cbc0cc575e9b053f41e72aa36bfa6b7f55e3529ffa43161b97fa")
        self.assertEqual(
            Download(
                url="https://crates.io/api/v1/crates/autocfg/1.1.0/download",
                filename="autocfg_1.1.0.crate",
                sha256="d468802bab17cbc0cc575e9b053f41e72aa36bfa6b7f55e3529ffa43161b97fa",
            ),
            package_version.to_download(),
        )


class GetPackageVersionsTestCase(TestCase):
    def test_get_package_versions(self) -> None:
        with get_from_url(CRYPTOGRAPHY__42_0_0__CARGO_LOCK) as path:
            with mock.patch.object(cargo_tools.logger, "warning") as warning_mock:
                package_versions = list(cargo_tools.get_package_versions(path))
        self.assertEqual(
            [
                PackageVersion(name="asn1", version="0.15.5", checksum="ae3ecbce89a22627b5e8e6e11d69715617138290289e385cde773b1fe50befdb"),
                PackageVersion(name="asn1_derive", version="0.15.5", checksum="861af988fac460ac69a09f41e6217a8fb9178797b76fcc9478444be6a59be19c"),
                PackageVersion(name="autocfg", version="1.1.0", checksum="d468802bab17cbc0cc575e9b053f41e72aa36bfa6b7f55e3529ffa43161b97fa"),
                PackageVersion(name="base64", version="0.21.7", checksum="9d297deb1925b89f2ccc13d7635fa0714f12c87adce1c75356b39ca9b7178567"),
                PackageVersion(name="bitflags", version="1.3.2", checksum="bef38d45163c2f1dde094a7dfd33ccf595c92905c8f8f4fdc18d06fb1037718a"),
                PackageVersion(name="bitflags", version="2.4.2", checksum="ed570934406eb16438a4e976b1b4500774099c13b8cb96eec99f620f05090ddf"),
                PackageVersion(name="cc", version="1.0.83", checksum="f1174fb0b6ec23863f8b971027804a42614e347eafb0a95bf0b12cdae21fc4d0"),
                PackageVersion(name="cfg-if", version="1.0.0", checksum="baf1de4339761588bc0619e3cbc0120ee582ebb74b53b4efbf79117bd2da40fd"),
                PackageVersion(name="foreign-types", version="0.3.2", checksum="f6f339eb8adc052cd2ca78910fda869aefa38d22d5cb648e6485e4d3fc06f3b1"),
                PackageVersion(name="foreign-types-shared", version="0.1.1", checksum="00b0228411908ca8685dba7fc2cdd70ec9990a6e753e89b6ac91a84c40fbaf4b"),
                PackageVersion(name="heck", version="0.4.1", checksum="95505c38b4572b2d910cecb0281560f54b440a19336cbbcb27bf6ce6adc6f5a8"),
                PackageVersion(name="indoc", version="2.0.4", checksum="1e186cfbae8084e513daff4240b4797e342f988cecda4fb6c939150f96315fd8"),
                PackageVersion(name="libc", version="0.2.152", checksum="13e3bf6590cbc649f4d1a3eefc9d5d6eb746f5200ffb04e5e142700b8faa56e7"),
                PackageVersion(name="lock_api", version="0.4.11", checksum="3c168f8615b12bc01f9c17e2eb0cc07dcae1940121185446edc3744920e8ef45"),
                PackageVersion(name="memoffset", version="0.9.0", checksum="5a634b1c61a95585bd15607c6ab0c4e5b226e695ff2800ba0cdccddf208c406c"),
                PackageVersion(name="once_cell", version="1.19.0", checksum="3fdb12b2476b595f9358c5161aa467c2438859caa136dec86c26fdd2efe17b92"),
                PackageVersion(name="openssl", version="0.10.63", checksum="15c9d69dd87a29568d4d017cfe8ec518706046a05184e5aea92d0af890b803c8"),
                PackageVersion(name="openssl-macros", version="0.1.1", checksum="a948666b637a0f465e8564c73e89d4dde00d72d4d473cc972f390fc3dcee7d9c"),
                PackageVersion(name="openssl-sys", version="0.9.99", checksum="22e1bf214306098e4832460f797824c05d25aacdf896f64a985fb0fd992454ae"),
                PackageVersion(name="parking_lot", version="0.12.1", checksum="3742b2c103b9f06bc9fff0a37ff4912935851bee6d36f3c02bcc755bcfec228f"),
                PackageVersion(name="parking_lot_core", version="0.9.9", checksum="4c42a9226546d68acdd9c0a280d17ce19bfe27a46bf68784e4066115788d008e"),
                PackageVersion(name="pem", version="3.0.3", checksum="1b8fcc794035347fb64beda2d3b462595dd2753e3f268d89c5aae77e8cf2c310"),
                PackageVersion(name="pkg-config", version="0.3.29", checksum="2900ede94e305130c13ddd391e0ab7cbaeb783945ae07a279c268cb05109c6cb"),
                PackageVersion(name="proc-macro2", version="1.0.78", checksum="e2422ad645d89c99f8f3e6b88a9fdeca7fabeac836b1002371c4367c8f984aae"),
                PackageVersion(name="pyo3", version="0.20.2", checksum="9a89dc7a5850d0e983be1ec2a463a171d20990487c3cfcd68b5363f1ee3d6fe0"),
                PackageVersion(name="pyo3-build-config", version="0.20.2", checksum="07426f0d8fe5a601f26293f300afd1a7b1ed5e78b2a705870c5f30893c5163be"),
                PackageVersion(name="pyo3-ffi", version="0.20.2", checksum="dbb7dec17e17766b46bca4f1a4215a85006b4c2ecde122076c562dd058da6cf1"),
                PackageVersion(name="pyo3-macros", version="0.20.2", checksum="05f738b4e40d50b5711957f142878cfa0f28e054aa0ebdfc3fd137a843f74ed3"),
                PackageVersion(name="pyo3-macros-backend", version="0.20.2", checksum="0fc910d4851847827daf9d6cdd4a823fbdaab5b8818325c5e97a86da79e8881f"),
                PackageVersion(name="quote", version="1.0.35", checksum="291ec9ab5efd934aaf503a6466c5d5251535d108ee747472c3977cc5acc868ef"),
                PackageVersion(name="redox_syscall", version="0.4.1", checksum="4722d768eff46b75989dd134e5c353f0d6296e5aaa3132e776cbdb56be7731aa"),
                PackageVersion(name="scopeguard", version="1.2.0", checksum="94143f37725109f92c262ed2cf5e59bce7498c01bcc1502d7b9afe439a4e9f49"),
                PackageVersion(name="self_cell", version="1.0.3", checksum="58bf37232d3bb9a2c4e641ca2a11d83b5062066f88df7fed36c28772046d65ba"),
                PackageVersion(name="smallvec", version="1.13.1", checksum="e6ecd384b10a64542d77071bd64bd7b231f4ed5940fba55e98c3de13824cf3d7"),
                PackageVersion(name="syn", version="2.0.48", checksum="0f3531638e407dfc0814761abb7c00a5b54992b849452a0646b7f65c9f770f3f"),
                PackageVersion(name="target-lexicon", version="0.12.13", checksum="69758bda2e78f098e4ccb393021a0963bb3442eac05f135c30f61b7370bbafae"),
                PackageVersion(name="unicode-ident", version="1.0.12", checksum="3354b9ac3fae1ff6755cb6db53683adb661634f67557942dea4facebec0fee4b"),
                PackageVersion(name="unindent", version="0.2.3", checksum="c7de7d73e1754487cb58364ee906a499937a0dfabd86bcb980fa99ec8c8fa2ce"),
                PackageVersion(name="vcpkg", version="0.2.15", checksum="accd4ea62f7bb7a82fe23066fb0957d48ef677f6eeb8215f372f52e48bb32426"),
                PackageVersion(name="windows-targets", version="0.48.5", checksum="9a2fa6e2155d7247be68c096456083145c183cbbbc2764150dda45a87197940c"),
                PackageVersion(name="windows_aarch64_gnullvm", version="0.48.5", checksum="2b38e32f0abccf9987a4e3079dfb67dcd799fb61361e53e2882c3cbaf0d905d8"),
                PackageVersion(name="windows_aarch64_msvc", version="0.48.5", checksum="dc35310971f3b2dbbf3f0690a219f40e2d9afcf64f9ab7cc1be722937c26b4bc"),
                PackageVersion(name="windows_i686_gnu", version="0.48.5", checksum="a75915e7def60c94dcef72200b9a8e58e5091744960da64ec734a6c6e9b3743e"),
                PackageVersion(name="windows_i686_msvc", version="0.48.5", checksum="8f55c233f70c4b27f66c523580f78f1004e8b5a8b659e05a4eb49d4166cca406"),
                PackageVersion(name="windows_x86_64_gnu", version="0.48.5", checksum="53d40abd2583d23e4718fddf1ebec84dbff8381c07cae67ff7768bbf19c6718e"),
                PackageVersion(name="windows_x86_64_gnullvm", version="0.48.5", checksum="0b7b52767868a23d5bab768e390dc5f5c55825b6d30b86c844ff2dc7414044cc"),
                PackageVersion(name="windows_x86_64_msvc", version="0.48.5", checksum="ed94fce61571a4006852b7389a063ab983c02eb1bb37b47f8272ce92d06d9538"),
            ],
            package_versions,
        )

        warning_mock.assert_has_calls(
            [
                mock.call("Skipping %s", {"name": "cryptography-cffi", "version": "0.1.0", "dependencies": ["cc", "openssl-sys", "pyo3"]}),
                mock.call(
                    "Skipping %s",
                    {"name": "cryptography-key-parsing", "version": "0.1.0", "dependencies": ["asn1", "cfg-if", "cryptography-x509", "openssl", "openssl-sys"]},
                ),
                mock.call(
                    "Skipping %s",
                    {"name": "cryptography-openssl", "version": "0.1.0", "dependencies": ["foreign-types", "foreign-types-shared", "openssl", "openssl-sys"]},
                ),
                mock.call(
                    "Skipping %s",
                    {
                        "name": "cryptography-rust", "version": "0.1.0",
                        "dependencies": [
                            "asn1", "cc", "cfg-if", "cryptography-cffi", "cryptography-key-parsing", "cryptography-openssl", "cryptography-x509",
                            "cryptography-x509-verification", "foreign-types-shared", "once_cell", "openssl", "openssl-sys", "pem", "pyo3", "self_cell",
                        ],
                    },
                ),
                mock.call("Skipping %s", {"name": "cryptography-x509", "version": "0.1.0", "dependencies": ["asn1"]}),
                mock.call(
                    "Skipping %s",
                    {"name": "cryptography-x509-verification", "version": "0.1.0", "dependencies": ["asn1", "cryptography-x509", "once_cell", "pem"]},
                ),
            ],
            any_order=False,
        )
        self.assertEqual(6, warning_mock.call_count, warning_mock.call_args_list)


class DownloadFromLockFileTestCase(TestCase):
    def test_download_from_lock_file(self) -> None:
        package_versions = [
            PackageVersion(name="lock_api", version="0.4.11", checksum="3c168f8615b12bc01f9c17e2eb0cc07dcae1940121185446edc3744920e8ef45"),
            PackageVersion(name="memoffset", version="0.9.0", checksum="5a634b1c61a95585bd15607c6ab0c4e5b226e695ff2800ba0cdccddf208c406c"),
            PackageVersion(name="once_cell", version="1.19.0", checksum="3fdb12b2476b595f9358c5161aa467c2438859caa136dec86c26fdd2efe17b92"),
        ]
        lock_path = Path("/path/to/Cargo.lock")
        with mock.patch.object(cargo_tools, "get_package_versions", return_value=package_versions), \
                mock.patch("license_tools.utils.download_utils.download_one_file_per_second") as download_mock, \
                TemporaryDirectory() as directory:
            cargo_tools.download_from_lock_file(lock_path=lock_path, target_directory=directory)
        download_mock.assert_called_once_with(
            downloads=[package.to_download() for package in package_versions],
            directory=Path(directory),
        )



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

"""
Tools related to RPM files.
"""

from __future__ import annotations

import contextlib
import datetime
import logging
import stat
from enum import IntEnum, IntFlag
from pathlib import Path
from typing import Any, cast

import rpmfile  # type: ignore[import-untyped]

logger = logging.getLogger(__name__)
del logging


# Docs: https://rpm-software-management.github.io/rpm/manual/tags.html
_VERBOSE_HEADER_NAMES = {
    # Known names: https://github.com/srossross/rpmfile/blob/4cd4ae2bd191d3489c95dfa540da14585670adb5/rpmfile/headers.py#L12-L363
    "headersignatures": "Header Signatures",
    "headerimmutable": "Unmodified, Original Header Image",
    "headeri18ntable": "Header Translation Locales",
    "sigsize": "Size Of Header + Payload",
    "sigmd5": "MD5 Digest Of Header + Payload",
    "sigpgp": "OpenPGP RSA Signature Of Header + Payload",
    "siggpg": "OpenPGP DSA Signature Of Header + Payload",
    "pubkeys": "Public PGP Keys",
    "signature": "OpenPGP DSA Signature Of Header",
    "rsaheader": "OpenPGP RSA Signature Of Header",
    "md5": "SHA1 Digest Of Header",  # Wrong name: https://github.com/srossross/rpmfile/issues/26
    "longsigsize": "Header + Payload Size when > 4 GB",
    "longarchivesize": "Uncompressed Payload Size when > 4 GB",
    "sha256": "SHA256 Digest Of Header",
    "veritysignatures": "fsverity Signature (Base64-encoded)",
    "veritysignaturealgo": "fsverity Signature Algorithm ID",
    "pgp": "PGP Signature From Signature Header",
    "pgp5": "PGP5 Signature From Signature Header",
    "gpg": "GPG Signature From Signature Header",
    "payloadsize": "Payload Size",
    "reservedspace": "Reserved Space",
    "name": "Package Name",
    "version": "Package Version",
    "release": "Package Release",
    "serial": "Package Epoch",
    "summary": "One-line Summary",
    "description": "Multi-line Description",
    "buildtime": "Package Build Time",
    "buildhost": "Hostname Of Build System",
    "installtime": "Package Installation Time",
    "size": "Installed Package Size",
    "distribution": "Distribution Name",
    "vendor": "Package Vendor",
    "copyright": "License Of Contents",
    "packager": "Packager",
    "group": "Package Group",
    "changelog": "Changelog",
    "source": "Source File Names",
    "patch": "Patch File Names",
    "url": "Upstream URL",
    "os": "Operating System",
    "arch": "Architecture",
    "prein": "Pre-installation Script",
    "postin": "Post-installation Script",
    "preun": "Pre-uninstallation Script",
    "postun": "Post-uninstallation Script",
    "filesizes": "File Sizes (when all < 4 GB)",
    "filestates": "Per-file Installation Status",
    "filemodes": "Unix Files Modes",
    "filerdevs": "Device IDs (of device files)",
    "filemtimes": "File Modification Timestamps",
    "filemd5s": "File Hash",
    "filelinktos": "File Symlink Targets",
    "fileflags": "File Flags",
    "fileusername": "File Unix Owner Names",
    "filegroupname": "File Unix Group Names",
    "sourcerpm": "Source RPM Filename",
    "fileverifyflags": "File Verification Flags",
    "archivesize": "Uncompressed Payload Size",
    "provides": "Provided Names",
    "requireflags": "Required Names (Flags)",
    "requirename": "Required Names",
    "requireversion": "Required Names (Versions)",
    "nosource": "Source Number Without Source",
    "nopatch": "Patch Number Without Source",
    "conflictflags": "Conflicting Names (Flags)",
    "conflictname": "Conflicting Names",
    "conflictversion": "Conflicting Names (Versions)",
    "excludearch": "Excluded Architecture",
    "excludeos": "Excluded Operating System",
    "exclusivearch": "Exclusive Architecture",
    "exclusiveos": "Exclusive Operating System",
    "rpmversion": "RPM Version For Building",
    "triggerscripts": "Trigger Scripts",
    "triggername": "Trigger Name",
    "triggerversion": "Trigger Version",
    "triggerflags": "Trigger Flags",
    "triggerindex": "Trigger Index",
    "verifyscript": "Verification Script",
    "changelogtime": "Changelog Timestamps",
    "authors": "Changelog Authors",
    "comments": "Changelog Texts",
    "preinprog": "Pre-installation Interpreter + Arguments",
    "postinprog": "Post-installation Interpreter + Arguments",
    "preunprog": "Pre-uninstallation Interpreter + Arguments",
    "postunprog": "Post-uninstallation Interpreter + Arguments",
    "buildarchs": "Possible Architectures",
    "obsoletes": "Obsolete Names",
    "verifyscriptprog": "Verification Interpreter + Arguments",
    "triggerscriptprog": "Trigger Script Interpreter + Arguments",
    "docdir": "Documentation Directory",
    "cookie": "Cookie For Build Operation",
    "filedevices": "File Devices",
    "fileinodes": "Abstract File Inode Numbers",
    "filelangs": "File Languages",
    "prefixes": "Relocatable Prefixes",
    "instprefixes": "Installation Prefixes",
    "sourcepackage": "Source RPM",
    "provideflags": "Provided Names (Flags)",
    "provideversion": "Provided Names (Versions)",
    "obsoleteflags": "Obsolete Names (Flags)",
    "obsoleteversion": "Obsolete Names (Versions)",
    "dirindexes": "Index Into Directory Names For Basenames",
    "basenames": "Basenames",
    "dirnames": "Directory Names",
    "origdirindexes": "Original Directory Indexes",
    "origbasenames": "Original Basenames",
    "origdirnames": "Original Directory Names",
    "optflags": "%{optflags} Value During Build",
    "disturl": "Distribution-specific URL",
    "archive_format": "Payload Format",
    "archive_compression": "Payload Compressor Name",
    "payloadflags": "Payload Compressor Level",
    "installcolor": "'Color' Of Package Installation Transaction",
    "installtid": "Package Installation Transaction ID",
    "target": "Package Platform",
    "filecolors": "File 'Color'",
    "fileclass": "Index Into Class Dictionary",
    "classdict": "Class Dictionary (File Class libmagic Entries)",
    "filedependsx": "Index Into File Dependencies Dictionary Denoting Start Of File's Dependencies",
    "filedependsn": "Number Of File Dependencies in Dependencies Dictionary",
    "dependsdict": "File Dependencies Dictionary",
    "sourcepkgid": "Source Package Identifier",
    "policies": "Policies",
    "pretrans": "Pre-transaction Script",
    "posttrans": "Post-transaction Script",
    "pretransprog": "Pre-transaction Interpreter + Arguments",
    "posttransprog": "Post-transaction Interpreter + Arguments",
    "disttag": "Distribution Acronym",
    "priority": "Priority",
    "cvsid": "CVS ID",
    "dbinstance": "Installed Package Header ID",
    "nvra": "Formatted name-version-release.arch Package String",
    "filenames": "Per File Paths Contained in the Package",
    "fileprovide": "Per File Dependency Capabilities Provided by the Files",
    "filerequire": "Per File Dependency Capabilities Required by the Files",
    "triggerconds": "Formatted Trigger Condition Information",
    "triggertype": "Formatted Trigger Type Information",
    "origfilenames": "Original Filenames (in relocated packages)",
    "longfilesizes": "File Size (when files > 4 GB)",
    "longsize": "Installed Package Size (when > 4 GB)",
    "filecaps": "Textual Representation of File Capabilities",
    "filedigestalgo": "File Digest Algorithm",
    "bugurl": "Bug Tracker URL",
    "evr": "Formatted epoch:version-release String",
    "nvr": "Formatted name-version-release String",
    "nevr": "Formatted name-epoch:version-release String",
    "nevra": "Formatted name-epoch:version-release.arch String",
    "headercolor": "Header 'Color' Calculated From File Colors",
    "verbose": "Verbose Mode",
    "epochnum": "Package Epoch (numeric)",
    "preinflags": "Pre-installation Flags",
    "postinflags": "Post-installation Flags",
    "preunflags": "Pre-uninstallation Flags",
    "postunflags": "Post-uninstallation Flags",
    "pretransflags": "Pre-transaction Flags",
    "posttransflags": "Post-transaction Flags",
    "verifyscriptflags": "Verification Script Flags",
    "triggerscriptflags": "Trigger Script Flags",
    "policynames": "Policy Names",
    "policytypes": "Policy Types",
    "policytypesindexes": "Policy Types Indexes",
    "policyflags": "Policy Flags",  # TODO: Provide IntFlag mapping.
    "vcs": "Upstream Source Code VCS Location",
    "ordername": "Order Names",
    "orderversion": "Order Names (Versions)",
    "orderflags": "Order Names (Flags)",
    "instfilenames": "Per File Paths Installed From The Package",
    "requirenevrs": "Formatted `name [op version]` Required Dependency Strings",
    "providenevrs": "Formatted `name [op version]` Provided Dependency Strings",
    "obsoletenevrs": "Formatted `name [op version]` Obsolete Dependency Strings",
    "conflictnevrs": "Formatted `name [op version]` Conflicting Dependency Strings",
    "filenlinks": "Per File Hardlink Number",
    "recommendname": "Recommended Names",
    "recommendversion": "Recommended Names (Versions)",
    "recommendflags": "Recommended Names (Flags)",
    "suggestname": "Suggested Names",
    "suggestversion": "Suggested Names (Versions)",
    "suggestflags": "Suggested Flags (Versions)",
    "supplementname": "Supplementary Names",
    "supplementversion": "Supplementary Names (Versions)",
    "supplementflags": "Supplementary Names (Flags)",
    "enhancename": "Enhancement Names",
    "enhanceversion": "Enhancement Names (Versions)",
    "enhanceflags": "Enhancement Names (Flags)",
    "recommendnevrs": "Formatted `name [op version]` Recommended Dependency Strings",
    "suggestnevrs": "Formatted `name [op version]` Suggested Dependency Strings",
    "supplementnevrs": "Formatted `name [op version]` Supplementary Dependency Strings",
    "enhancenevrs": "Formatted `name [op version]` Enhancement Dependency Strings",
    "encoding": "Header String Data Encoding",
    "filetriggerscripts": "File Trigger Scripts",
    "filetriggerscriptprog": "File Trigger Scripts Interpreters + Arguments",
    "filetriggerscriptflags": "File Trigger Scripts Flags",
    "filetriggername": "File Trigger Names",
    "filetriggerindex": "File Trigger Indexes",
    "filetriggerversion": "File Trigger Versions",
    "filetriggerflags": "File Trigger Flags",
    "transfiletriggerscripts": "Transaction File Trigger Scripts",
    "transfiletriggerscriptprog": "Transaction File Trigger Scripts Interpreters + Arguments",
    "transfiletriggerscriptflags": "Transaction File Trigger Flags",
    "transfiletriggername": "Transaction File Trigger Names",
    "transfiletriggerindex": "Transaction File Trigger Indexes",
    "transfiletriggerversion": "Transaction File Trigger Versions",
    "transfiletriggerflags": "Transaction File Trigger Flags",
    "filetriggerpriorities": "File Trigger Priorities",
    "transfiletriggerpriorities": "Transaction File Trigger Priorities",
    "filetriggerconds": "Formatted File Trigger Condition Information",
    "filetriggertype": "Formatted File Trigger Type Information",
    "transfiletriggerconds": "Formatted Transaction File Trigger Condition Information",
    "transfiletriggertype": "Formatted Transaction File Trigger Type Information",
    "filesignatures": "IMA Signatures (hex encoded)",
    "filesignaturelength": "IMA Signature Length",
    "payloaddigest": "Cryptographic Digest Of Compressed Payload",
    "payloaddigestalgo": "Payload Digest Algorithm",
    "modularitylabel": "Modularity Label",
    "payloaddigestalt": "Cryptographic Digest Of Uncompressed Payload",
    "archsuffix": "Package File Arch Suffix",
    "spec": "Expanded And Parsed Spec Contents",
    "translationurl": "URL Of Upstream Translation Service/Repository",
    "upstreamreleases": "URL To Check For Newer Upstream Releases",
    "loaddigestalt": "Cryptographic Digest Of Uncompressed Payload",
    "upstreamleases": "URL To Check For Newer Upstream Releases",
    "sourcelicense": "Source License",
    "sysusers": "Formatted systemd-sysusers Line",
    "preuntrans": "Pre-uninstallation-transaction Script",
    "postuntrans": "Post-uninstallation-transaction Script",
    "preuntransprog": "Pre-uninstallation-transaction Interpreter + Arguments",
    "postuntransprog": "Post-uninstallation-transaction Interpreter + Arguments",
    "preuntransflags": "Pre-uninstallation-transaction Flags",
    "postuntransflags": "Post-uninstallation-transaction Flags",
}

# These headers are of no real use (for now), not properly documented or deprecated.
_HEADERS_TO_OMIT = {
    "autoinstalled",
    "autoprov",
    "autoreq",
    "autoreqprov",
    "badsha1_1",
    "badsha1_2",
    "blinkhdrid",
    "blinknevra",
    "blinkpkgid",
    "buildcpuclock",
    "buildconflicts",
    "buildenhances",
    "buildmacros",
    "buildobsoletes",
    "buildplatforms",
    "buildprereq",
    "buildprovides",
    "buildrequires",
    "buildroot",
    "buildsuggests",
    "cachectime",
    "cachepkgmtime",
    "cachepkgpath",
    "cachepkgsize",
    "capability",
    "collections",
    "conflictattrsx",
    "defaultprefix",
    "depattrsdict",
    "exclude",
    "exclusive",
    "filecontexts",
    "filedigestalgos",
    "filegids",
    "filetriggerin",
    "filetriggerpostun",
    "filetriggerun",
    "fileuids",
    "filexattrsx",
    "flinkhdrid",
    "flinknevra",
    "flinkpgkid",
    "fscontexts",
    "fsnames",
    "fssizes",
    "gif",
    "headerimage",
    "headerregions",
    "icon",
    "identity",
    "installprefix",
    "keywords",
    "lemd5_2",
    "mssfdomain",
    "mssfmanifest",
    "obsoleteattrsx",
    "oldenhancesflags",
    "oldenhancesnames",
    "oldenhancesversion",
    "oldorigfilenames",
    "oldsuggestsflags",
    "oldsuggestsname",
    "oldsuggestsversion",
    "packagecolor",
    "packageorigin"
    "packageprefcolor",
    "patchesflags",
    "patchesname",
    "patchesversion",
    "prereq",
    "provideattrsx",
    "recontexts",
    "removepathpostfixes",
    "removetid",
    "repotag",
    "requireattrsx",
    "rhnplatform",
    "root",
    "scriptmetrics",
    "scriptstates",
    "sig_base",
    "siglemd5_1",
    "siglemd5_2",
    "sigpgp5",
    "transfiletriggerin",
    "transfiletriggerpostun",
    "transfiletriggerun",
    "triggerin",
    "triggerun",
    "triggerpostun",
    "triggerprein",
    "variants",
    "xattrsdict",
    "xmajor",
    "xminor",
    "xpm",
}


def extract(archive_path: Path, target_path: Path) -> None:
    """
    Extract the given RPM file.

    :param archive_path: The RPM file to unpack.
    :param target_path: The directory to unpack to.
    """
    target_path_str = str(target_path)

    # See `rpmfile.cli` for the `extract` option.
    # This is a pathlib-based approach of the original implementation.
    #
    # Upstream code:
    # https://github.com/srossross/rpmfile/blob/c0498cd5173afb6fb0af9ed5c7d61335b7c9af0e/rpmfile/cli.py
    #
    # Original copyright:
    #
    # -------------------------------------------------------------------------
    #
    # Copyright (c) 2015 Sean Ross-Ross
    #
    # MIT License
    #
    # Permission is hereby granted, free of charge, to any person obtaining a copy
    # of this software and associated documentation files (the "Software"), to deal
    # in the Software without restriction, including without limitation the rights
    # to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    # copies of the Software, and to permit persons to whom the Software is
    # furnished to do so, subject to the following conditions:
    #
    # The above copyright notice and this permission notice shall be included in all
    # copies or substantial portions of the Software.
    #
    # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    # AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    # SOFTWARE.
    #
    # -------------------------------------------------------------------------
    #
    # The changed version can be used under either the MIT or the Apache-2.0 license,
    # depending on your preferences.
    with rpmfile.open(archive_path) as rpm_file:
        for rpm_info in rpm_file.getmembers():
            with rpm_file.extractfile(rpm_info.name) as file_object:
                directories = rpm_info.name.split("/")
                filename = directories.pop()
                if directories:
                    directories_path = target_path.joinpath(*directories).resolve()
                    if not str(directories_path).startswith(target_path_str):
                        raise ValueError(f"Attempted path traversal: {directories_path}")
                    if not directories_path.is_dir():
                        directories_path.mkdir(parents=True)
                else:
                    directories_path = target_path.resolve()
                target_file = directories_path / filename
                if not str(target_file).startswith(target_path_str):
                    raise ValueError(f"Attempted path traversal: {target_file}")
                target_file.write_bytes(file_object.read())


def get_headers(rpm_path: Path) -> dict[str, Any]:
    """
    Get the RPM headers.

    :param rpm_path: The RPM file to analyze.
    :return: The corresponding headers.
    """
    with rpmfile.open(rpm_path) as rpm_file:
        headers = rpm_file.headers
    return cast(dict[str, Any], headers)


class FileFlags(IntFlag):
    """
    Flags used to indicate the file types.
    """

    # https://refspecs.linuxbase.org/LSB_3.1.0/LSB-Core-generic/LSB-Core-generic/pkgformat.html, section 22.2.4.3.1
    # https://github.com/eclipse/packager/blob/e481c63dd76b112b3924d93e39ab6e791e60a41b/rpm/src/main/java/org/eclipse/packager/rpm/FileFlags.java
    CONFIG = 1 << 0
    DOC = 1 << 1
    DO_NOT_USE = 1 << 2
    MISSING_OK = 1 << 3
    NO_REPLACE = 1 << 4
    SPECFILE = 1 << 5
    GHOST = 1 << 6
    LICENSE = 1 << 7
    README = 1 << 8
    EXCLUDE = 1 << 9
    IGNORE = 1 << 10
    PUBKEY = 1 << 11
    ARTIFACT = 1 << 12


class VerifyFlags(IntFlag):
    """
    Flags used to indicate how to verify the file.
    """

    # https://github.com/eclipse/packager/blob/e481c63dd76b112b3924d93e39ab6e791e60a41b/rpm/src/main/java/org/eclipse/packager/rpm/VerifyFlags.java
    MD5 = 1 << 0
    SIZE = 1 << 1
    LINK_TO = 1 << 2
    USER = 1 << 3
    GROUP = 1 << 4
    MTIME = 1 << 5
    MODE = 1 << 6
    RDEV = 1 << 7
    CAPS = 1 << 8


class DependencyFlags(IntFlag):
    """
    Flags used to indicate how to handle dependency relationships.
    """

    # https://refspecs.linuxbase.org/LSB_3.1.0/LSB-Core-generic/LSB-Core-generic/pkgformat.html, section 22.2.4.4.2
    # https://github.com/rpm-software-management/rpm/blob/f1b68c9e7672a2af8251d611f61c4ab17fcd3ea8/include/rpm/rpmds.h#L22-L53
    ANY = 0
    LESS = 1 << 1
    GREATER = 1 << 2
    EQUAL = 1 << 3
    UNUSED4 = 1 << 4
    POSTTRANS = 1 << 5
    PREREQ = 1 << 6
    PRETRANS = 1 << 7
    INTERP = 1 << 8
    SCRIPT_PRE = 1 << 9
    SCRIPT_POST = 1 << 10
    SCRIPT_PREUN = 1 << 11
    SCRIPT_POSTUN = 1 << 12
    SCRIPT_VERIFY = 1 << 13
    FIND_REQUIRES = 1 << 14
    FIND_PROVIDES = 1 << 15
    TRIGGERIN = 1 << 16
    TRIGGERUN = 1 << 17
    TRIGGERPOSTUN = 1 << 18
    MISSINGOK = 1 << 19
    PREUNTRANS = 1 << 20
    POSTUNTRANS = 1 << 21
    UNUSED22 = 1 << 22
    UNUSED23 = 1 << 23
    RPMLIB = 1 << 24
    TRIGGERPREIN = 1 << 25
    KEYRING = 1 << 26
    UNUSED27 = 1 << 27
    CONFIG = 1 << 28
    META = 1 << 29


class FileColor(IntEnum):
    """
    Enumeration of file "color"/types.
    """

    ELF_32_BIT = 1
    ELF_64_BIT = 2
    OTHER = 0


class FileDigestAlgorithm(IntEnum):
    """
    Enumeration of allowed file digest algorithms.
    """

    # https://docs.rs/rpm-rs/latest/rpm/enum.FileDigestAlgorithm.html
    MD5 = 0
    SHA1 = 1
    MD2 = 2
    HAVAL_5_160 = 3
    RIPEMD160 = 4
    TIGER_192 = 5
    SHA256 = 6
    SHA384 = 7
    SHA512 = 8
    SHA224 = 9


class FileModes(IntEnum):
    """
    Enumeration of known file modes.
    """

    # File types.
    IS_DIRECTORY = stat.S_IFDIR
    IS_CHARACTER_DEVICE = stat.S_IFCHR
    IS_BLOCK_DEVICE = stat.S_IFBLK
    IS_REGULAR_FILE = stat.S_IFREG
    IS_FIFO_NAMED_PIPE = stat.S_IFIFO
    IS_SYMBOLIC_LINK = stat.S_IFLNK
    IS_SOCKET_FILE = stat.S_IFSOCK

    # Permissions.
    SET_UID_BIT = stat.S_ISUID
    SET_GID_BIT = stat.S_ISGID  # The same as file locking enforcement.
    STICKY_BIT = stat.S_ISVTX
    READ_BY_OWNER_ALTERNATIVE = stat.S_IREAD
    WRITE_BY_OWNER_ALTERNATIVE = stat.S_IWRITE
    EXECUTE_BY_OWNER_ALTERNATIVE = stat.S_IEXEC
    READ_WRITE_EXECUTE_BY_OWNER = stat.S_IRWXU
    READ_BY_OWNER = stat.S_IRUSR
    WRITE_BY_OWNER = stat.S_IWUSR
    EXECUTE_BY_OWNER = stat.S_IXUSR
    READ_WRITE_EXECUTE_BY_GROUP = stat.S_IRWXG
    READ_BY_GROUP = stat.S_IRGRP
    WRITE_BY_GROUP = stat.S_IWGRP
    EXECUTE_BY_GROUP = stat.S_IXGRP
    READ_WRITE_EXECUTE_BY_OTHERS = stat.S_IRWXO
    READ_BY_OTHERS = stat.S_IROTH
    WRITE_BY_OTHERS = stat.S_IWOTH
    EXECUTE_BY_OTHERS = stat.S_IXOTH

    @classmethod
    def make_verbose(cls, mode: int) -> list[str]:
        """
        Convert the given mode value into a verbose representation.

        :param mode: The mode to convert.
        :return: The corresponding named constants matching the given mode.
        """
        st_type = stat.S_IFMT(mode)
        st_mode = stat.S_IMODE(mode)

        result = []
        for name, value in cls.__members__.items():
            if name.startswith("IS_"):
                if st_type == value:
                    result.append(name)
            elif st_mode & value == value:
                result.append(name)
        return result


def _convert_header_value(key: str, value: Any) -> Any:
    """
    Convert the given header value to the appropriate type.

    :param key: The header key.
    :param value: The header value.
    :return: The converted value.
    """
    # Attempt to decode bytes.
    if isinstance(value, bytes):
        with contextlib.suppress(UnicodeDecodeError):
            value = value.decode("UTF-8")

    # Do not differentiate between lists and tuples.
    if isinstance(value, tuple):
        value = list(value)

    # Attempt to decode lists of strings.
    if isinstance(value, list) and value and isinstance(value[0], bytes):
        with contextlib.suppress(UnicodeDecodeError):
            value = [entry.decode("UTF-8") for entry in value]

    # Convert UTC timestamps.
    if key == "buildtime":
        assert isinstance(value, int)
        return datetime.datetime.fromtimestamp(value, tz=datetime.timezone.utc)
    if key == "installtime":
        if isinstance(value, int):
            return datetime.datetime.fromtimestamp(value, tz=datetime.timezone.utc)
        return None
    if key in {"filemtimes", "changelogtime"}:
        return [datetime.datetime.fromtimestamp(mtime, tz=datetime.timezone.utc) for mtime in value]

    # Convert flags/enums.
    if key == "fileflags":
        return [FileFlags(flags) for flags in value]
    if key == "fileverifyflags":
        return [VerifyFlags(flags) for flags in value]
    if key in {"requireflags", "provideflags", "conflictflags", "obsoleteflags"}:
        if isinstance(value, int):
            return DependencyFlags(value)
        return [DependencyFlags(flags) for flags in value]
    if key == "filecolors":
        return [FileColor(color) for color in value]
    if key in {"filedigestalgo", "payloaddigestalgo"}:
        return FileDigestAlgorithm(value)
    if key == "filemodes":
        # Emulate the regular enumeration display.
        return ["<FileModes." + "|".join(FileModes.make_verbose(mode)) + f": {mode}>" for mode in value]

    # TODO: These are just null bytes. Is this value correct?
    if key == "reservedspace":
        return len(value)

    # Keep the value without further conversion.
    return value


def get_nice_headers(rpm_path: Path) -> dict[str, Any]:
    """
    Get the RPM headers, but with nicer values and verbose names as keys.

    :param rpm_path: The RPM file to analyze.
    :return: The corresponding headers.
    """
    result: dict[str, Any] = {}
    for key, value in get_headers(rpm_path).items():
        if key in _HEADERS_TO_OMIT:
            logger.warning("Omitting key %s ...", key)
            continue
        if key not in _VERBOSE_HEADER_NAMES:
            logger.warning("Detected unknown key %s. Skipping it ...", key)
            continue
        verbose_key = _VERBOSE_HEADER_NAMES[key]
        transformed_value = _convert_header_value(key, value)
        result[verbose_key] = transformed_value
    return result


def check_rpm_headers(path: Path) -> str | None:
    """
    Render the relevant header details for the given RPM.

    :param path: The RPM path.
    :return: `None` if no results could be determined, otherwise the rendered
             dictionary-like representation of the header section which usually
             holds the copyright data.
    """
    header_data = get_nice_headers(path)
    if not header_data:
        return None
    maximum_length = max(map(len, header_data.keys()))

    def display(v: Any) -> str:
        if isinstance(v, (IntEnum, IntFlag, datetime.datetime)):
            return repr(v)
        return str(v)

    return "\n".join(
        f"{key:>{maximum_length}}: {display(value)}" for key, value in header_data.items()
    )



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

"""
Rendering utilities.
"""

from __future__ import annotations

from typing import Any


def render_dictionary(dictionary: dict[str, Any], verbose_names_mapping: dict[str, str], multi_value_keys: set[str]) -> str:
    """
    Render the given dictionary as string.

    :param dictionary: The dictionary to render.
    :param verbose_names_mapping: The mapping dictionary to use for the keys.
                                  Keys not available inside this dictionary will be skipped.
    :param multi_value_keys: Dictionary keys which could have multiple values.
    """
    maximum_length = max(map(len, verbose_names_mapping.values()))
    rendered = []
    for key, verbose_name in verbose_names_mapping.items():
        if key not in dictionary:
            continue
        value = dictionary[key]
        if key in multi_value_keys and isinstance(value, (list, set, tuple)):
            if isinstance(value, tuple):
                value = list(value)
            if len(value) == 1:
                value = value.pop()
                rendered.append(f"{verbose_name:>{maximum_length}}: {value}")
            elif not value:
                rendered.append(f"{verbose_name:>{maximum_length}}:")
            else:
                value = "\n" + "\n".join(" " * maximum_length + f"   * {x}" for x in sorted(value))
                rendered.append(f"{verbose_name:>{maximum_length}}:{value}")
        else:
            rendered.append(f"{verbose_name:>{maximum_length}}: {value}")
    return "\n".join(rendered)



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

from __future__ import annotations

import sys
from pathlib import Path
from tempfile import TemporaryDirectory
from unittest import TestCase

from license_tools.tools import rpm_tools
from license_tools.utils.path_utils import get_files_from_directory
from tests import get_from_url
from tests.data import LIBAIO1__0_3_109_1_25__RPM, LIBAIO1__0_3_109_1_25__SRC_RPM


class ExtractTestCase(TestCase):
    def test_unpack_rpm_file(self) -> None:
        with get_from_url(LIBAIO1__0_3_109_1_25__RPM) as path, TemporaryDirectory() as tempdir:
            directory = Path(tempdir)
            rpm_tools.extract(
                archive_path=path, target_path=directory,
            )
            actual = [x[1] for x in get_files_from_directory(directory)]
            self.assertEqual(
                [
                    "lib64/libaio.so.1",
                    "lib64/libaio.so.1.0.1",
                    "usr/share/doc/packages/libaio1/COPYING",
                    "usr/share/doc/packages/libaio1/TODO",
                ],
                actual,
            )


class FileModesTestCase(TestCase):
    def test_make_verbose(self) -> None:
        modes = [41471, 33261, 16877, 33188]
        expected_by_mode = [
            [
                "IS_SYMBOLIC_LINK",
                "READ_BY_OWNER_ALTERNATIVE",
                "WRITE_BY_OWNER_ALTERNATIVE",
                "EXECUTE_BY_OWNER_ALTERNATIVE",
                "READ_WRITE_EXECUTE_BY_OWNER",
                "READ_BY_OWNER",
                "WRITE_BY_OWNER",
                "EXECUTE_BY_OWNER",
                "READ_WRITE_EXECUTE_BY_GROUP",
                "READ_BY_GROUP",
                "WRITE_BY_GROUP",
                "EXECUTE_BY_GROUP",
                "READ_WRITE_EXECUTE_BY_OTHERS",
                "READ_BY_OTHERS",
                "WRITE_BY_OTHERS",
                "EXECUTE_BY_OTHERS",
            ],
            [
                "IS_REGULAR_FILE",
                "READ_BY_OWNER_ALTERNATIVE",
                "WRITE_BY_OWNER_ALTERNATIVE",
                "EXECUTE_BY_OWNER_ALTERNATIVE",
                "READ_WRITE_EXECUTE_BY_OWNER",
                "READ_BY_OWNER",
                "WRITE_BY_OWNER",
                "EXECUTE_BY_OWNER",
                "READ_BY_GROUP",
                "EXECUTE_BY_GROUP",
                "READ_BY_OTHERS",
                "EXECUTE_BY_OTHERS",
            ],
            [
                "IS_DIRECTORY",
                "READ_BY_OWNER_ALTERNATIVE",
                "WRITE_BY_OWNER_ALTERNATIVE",
                "EXECUTE_BY_OWNER_ALTERNATIVE",
                "READ_WRITE_EXECUTE_BY_OWNER",
                "READ_BY_OWNER",
                "WRITE_BY_OWNER",
                "EXECUTE_BY_OWNER",
                "READ_BY_GROUP",
                "EXECUTE_BY_GROUP",
                "READ_BY_OTHERS",
                "EXECUTE_BY_OTHERS",
            ],
            [
                "IS_REGULAR_FILE",
                "READ_BY_OWNER_ALTERNATIVE",
                "WRITE_BY_OWNER_ALTERNATIVE",
                "READ_BY_OWNER",
                "WRITE_BY_OWNER",
                "READ_BY_GROUP",
                "READ_BY_OTHERS",
            ],
        ]
        for mode, expected in zip(modes, expected_by_mode):
            with self.subTest(mode=mode):
                self.assertEqual(expected, rpm_tools.FileModes.make_verbose(mode))


class CheckRpmHeadersTestCase(TestCase):
    maxDiff = None

    def test_binary(self) -> None:
        if sys.version_info < (3, 11):
            file_flags = "[<FileFlags.0: 0>, <FileFlags.0: 0>, <FileFlags.0: 0>, <FileFlags.DOC: 2>, <FileFlags.DOC: 2>]"
            file_verification_flags = "[<VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>, <VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>, <VerifyFlags.0: 0>, <VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>, <VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>]"  # noqa: E501
            required_names_flags = "[<DependencyFlags.SCRIPT_POST|INTERP: 1280>, <DependencyFlags.SCRIPT_POSTUN|INTERP: 4352>, <DependencyFlags.FIND_REQUIRES: 16384>, <DependencyFlags.FIND_REQUIRES: 16384>, <DependencyFlags.RPMLIB|EQUAL|LESS: 16777226>, <DependencyFlags.RPMLIB|EQUAL|LESS: 16777226>, <DependencyFlags.RPMLIB|EQUAL|LESS: 16777226>, <DependencyFlags.RPMLIB|EQUAL|LESS: 16777226>]"  # noqa: E501
        else:
            file_flags = "[<FileFlags: 0>, <FileFlags: 0>, <FileFlags: 0>, <FileFlags.DOC: 2>, <FileFlags.DOC: 2>]"
            file_verification_flags = "[<VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>, <VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>, <VerifyFlags: 0>, <VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>, <VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>]"  # noqa: E501
            required_names_flags = "[<DependencyFlags.INTERP|SCRIPT_POST: 1280>, <DependencyFlags.INTERP|SCRIPT_POSTUN: 4352>, <DependencyFlags.FIND_REQUIRES: 16384>, <DependencyFlags.FIND_REQUIRES: 16384>, <DependencyFlags.LESS|EQUAL|RPMLIB: 16777226>, <DependencyFlags.LESS|EQUAL|RPMLIB: 16777226>, <DependencyFlags.LESS|EQUAL|RPMLIB: 16777226>, <DependencyFlags.LESS|EQUAL|RPMLIB: 16777226>]"  # noqa: E501

        with get_from_url(LIBAIO1__0_3_109_1_25__RPM) as rpm_path:
            results = rpm_tools.check_rpm_headers(rpm_path)
        self.assertEqual(
            r"""
                                                            Header Signatures: b'\x00\x00\x00>\x00\x00\x00\x07\xff\xff\xffp\x00\x00\x00\x10'
                                              OpenPGP RSA Signature Of Header: b'\x89\x01\x15\x03\x05\x00[\x08R$p\xaf\x9e\x819\xdb|\x82\x01\x08\x8a\xe2\x08\x00\xc2_\x1a\x1f\xc7E>\x9f\x96"(\x0eF4b>\xb4\x00\x99\xb1y\xab\xb1\xa8>\xe4\xe4\xeb\x99\xe2?D`\xd3\x0eS\xc7P{{\x88\x8d\xa3dB\xa3\xef\\{{\xa3\xf8\x06cg\xc1\x98\xea\x82u\xa4\xb9.r^o\x91\xe2o\x19\x1c\xcc$\xf9\xcd\x1b\x85\x98f[\xa2\xdd!3_\xde\x81\xea`\x99\xeb\x83Y\xd1\xfaw\x81\xf774\xd1\x11\xf6\xbf\x98\xb2\xb1\x84\xf1\x9e^l\x1eY\xc9\x9c\x1c\xed\xc8\xd3[X\xeb\xc7h7\x82V\xab\x80\xd7]\xb0\xdbC\x86\xe2\xc0\x9d\x86\xd0 %\x1e{{\x1c\xd394\x89\xabp8\x04GE\xf4Z4(\xeaoF\xec\xe1\x9c&\xd1\x17z\x8dC/\x08\xf0\x0e(\xcc9\x04\xddJ\xb3b\xe4)\x8b\'\xac\x15\xb6\x1a?\xb2\xd9\xdf\xbeJ\xf7\xd8\xfa~*\xaa\xa5S\x8c\xd8\x0c\xd1J\xec<j\xee\xf3\xab9\x10Fe\xaa4I\xf4;\xd4v\x83\xe9BMN\xff\xd3{{\xb2\xd1(Py"\x1f\'\'(\xc9?:\xb1\xee\xd9\x0f\x1f\x8c\x9d|P'
                                                        SHA1 Digest Of Header: dbeb3eb27337e5d21b569b6cc73165320c35164b
                                                      SHA256 Digest Of Header: 3c4d1b9d2da6a1223c2ed6b2187f6c9618982c9526405ab88112fde3a06fe03e
                                                       Installed Package Size: 32262
                                          PGP Signature From Signature Header: b'\x89\x01\x15\x03\x05\x00[\x08R$p\xaf\x9e\x819\xdb|\x82\x01\x08\xdb`\x07\xfe%\xfa\x0e\x97\xacl\xf7\x12\xb5\xf0h\xd8\x12q\x88\x008\xf0\xa7\xffJF\x91\xfb4\xb5<:\xf5\x84=\xbb\xeeFH]3\xf7{{\x87\xa9G\xa5\xeb\x0f*V\xcf\x97O\xea\x9f\xaf:v8\x90\xf2\xbd\x9dh\xf2\xee(\xf4\x8a\x0f8\x15\xa9\xd2;Op\x04a\x11\x11\xf1\xd2\xf0ir\xcaC\xa3\xe4\x84\xcd\xac\x91]\xf3\xed/\xc5\xf2H\x17[j\xc1\x95\x11\nX\x8d\x0f\xed\xb6\xbcy,\xcc\xd7\xbajh\x94o\xca\xe1\x10\x9ap\xd1\xa9\x9f^\xb8\x08\xe0j\x18\xc1!\x9f\xc0\xa5\xf8\xeb.Z\x81L\x82\x97\xf0.\xed\xe2UHu\xd0(\xfc\x0f\xab\xf3FvC\xe08K\x10\xdb\xf9\x08\xb0\x18 \xff!\xce\xe5\xb4\xd6\xec\xbf\x7f\xe2\xb2\xa1\xc7\x92\xc3\n\x8bA\xdd\xfdf\x18*\xc8\x98\xc4\xcf\x15%\x1c\x9f\xc7\x16\xb0@:9\xbd\xc3|\xc2}}\xd2i\xa3\xe2t\x98\x8e\np\x1eQ\x9dd\xb3K\x1f\x03\x9eA\x92O\xcb\x8f\xa1\xdf\x17~1\x94\xcd\xe7c\xd6\xd0\xb1O\xe5* \xdd\x0f'
                                               MD5 Digest Of Header + Payload: b'\x983\xff\x1f\x06\xea\xc3\xd5\xf1\xec\xf9\x1b\xba\xd95\r'
                                                                 Payload Size: 33116
                                                               Reserved Space: 4128
                                            Unmodified, Original Header Image: b'\x00\x00\x00?\x00\x00\x00\x07\xff\xff\xfb\xd0\x00\x00\x00\x10'
                                                   Header Translation Locales: ['C']
                                                                 Package Name: libaio1
                                                              Package Version: 0.3.109
                                                              Package Release: 1.25
                                                             One-line Summary: Linux-Native Asynchronous I/O Access Library
                                                       Multi-line Description: The Linux-native asynchronous I/O facility ("async I/O", or "aio") has
a richer API and capability set than the simple POSIX async I/O
facility. This library provides the Linux-native API for async I/O. The
POSIX async I/O facility requires this library to provide
kernel-accelerated async I/O capabilities, as do applications that
require the Linux-native async I/O API.
                                                           Package Build Time: datetime.datetime(2018, 5, 25, 18, 12, 44, tzinfo=datetime.timezone.utc)
                                                     Hostname Of Build System: sheep53
                                                            Distribution Name: SUSE Linux Enterprise 15
                                                               Package Vendor: SUSE LLC <https://www.suse.com/>
                                                          License Of Contents: LGPL-2.1+
                                                                     Packager: https://www.suse.com/
                                                                Package Group: System/Libraries
                                                                 Upstream URL: http://kernel.org/pub/linux/libs/aio/
                                                             Operating System: linux
                                                                 Architecture: x86_64
                                                 File Sizes (when all < 4 GB): [15, 5608, 0, 26532, 122]
                                                             Unix Files Modes: ['<FileModes.IS_SYMBOLIC_LINK|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|EXECUTE_BY_OWNER_ALTERNATIVE|READ_WRITE_EXECUTE_BY_OWNER|READ_BY_OWNER|WRITE_BY_OWNER|EXECUTE_BY_OWNER|READ_WRITE_EXECUTE_BY_GROUP|READ_BY_GROUP|WRITE_BY_GROUP|EXECUTE_BY_GROUP|READ_WRITE_EXECUTE_BY_OTHERS|READ_BY_OTHERS|WRITE_BY_OTHERS|EXECUTE_BY_OTHERS: 41471>', '<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|EXECUTE_BY_OWNER_ALTERNATIVE|READ_WRITE_EXECUTE_BY_OWNER|READ_BY_OWNER|WRITE_BY_OWNER|EXECUTE_BY_OWNER|READ_BY_GROUP|EXECUTE_BY_GROUP|READ_BY_OTHERS|EXECUTE_BY_OTHERS: 33261>', '<FileModes.IS_DIRECTORY|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|EXECUTE_BY_OWNER_ALTERNATIVE|READ_WRITE_EXECUTE_BY_OWNER|READ_BY_OWNER|WRITE_BY_OWNER|EXECUTE_BY_OWNER|READ_BY_GROUP|EXECUTE_BY_GROUP|READ_BY_OTHERS|EXECUTE_BY_OTHERS: 16877>', '<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|READ_BY_OWNER|WRITE_BY_OWNER|READ_BY_GROUP|READ_BY_OTHERS: 33188>', '<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|READ_BY_OWNER|WRITE_BY_OWNER|READ_BY_GROUP|READ_BY_OTHERS: 33188>']
                                                 Device IDs (of device files): [0, 0, 0, 0, 0]
                                                 File Modification Timestamps: [datetime.datetime(2018, 5, 25, 18, 12, 43, tzinfo=datetime.timezone.utc), datetime.datetime(2018, 5, 25, 18, 12, 43, tzinfo=datetime.timezone.utc), datetime.datetime(2018, 5, 25, 18, 12, 44, tzinfo=datetime.timezone.utc), datetime.datetime(2009, 10, 9, 18, 17, 2, tzinfo=datetime.timezone.utc), datetime.datetime(2009, 10, 9, 18, 17, 2, tzinfo=datetime.timezone.utc)]
                                                                    File Hash: ['', 'bf8977f717cdf65b34fdad5e00daec89bdbc9a6ac512887c5cf2e78ff7cc7191', '', '5bbcbb737e60fe9deba08ecbd00920cfcc3403ba2e534c64fdeea49d6bb87509', 'f7b7efb8cb7bf444fe0f8c97f0989308e0a6d54e888c87db501494973d2c656b']
                                                         File Symlink Targets: ['libaio.so.1.0.1', '', '', '', '']
                                                                   File Flags: {file_flags}
                                                        File Unix Owner Names: ['root', 'root', 'root', 'root', 'root']
                                                        File Unix Group Names: ['root', 'root', 'root', 'root', 'root']
                                                          Source RPM Filename: libaio-0.3.109-1.25.src.rpm
                                                      File Verification Flags: {file_verification_flags}
                                                               Provided Names: ['libaio', 'libaio.so.1()(64bit)', 'libaio.so.1(LIBAIO_0.1)(64bit)', 'libaio.so.1(LIBAIO_0.4)(64bit)', 'libaio1', 'libaio1(x86-64)']
                                                       Required Names (Flags): {required_names_flags}
                                                               Required Names: ['/sbin/ldconfig', '/sbin/ldconfig', 'libc.so.6()(64bit)', 'libc.so.6(GLIBC_2.4)(64bit)', 'rpmlib(CompressedFileNames)', 'rpmlib(FileDigests)', 'rpmlib(PayloadFilesHavePrefix)', 'rpmlib(PayloadIsXz)']
                                                    Required Names (Versions): ['', '', '', '', '3.0.4-1', '4.6.0-1', '4.0-1', '5.2-1']
                                                     RPM Version For Building: 4.14.1
                                                         Changelog Timestamps: [datetime.datetime(2016, 4, 17, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2014, 8, 26, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2013, 3, 1, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2012, 2, 17, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2012, 2, 16, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2012, 2, 13, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2011, 11, 28, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2011, 11, 28, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2011, 10, 5, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2011, 9, 30, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2011, 3, 15, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2010, 2, 12, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2010, 1, 23, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2009, 8, 2, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2009, 3, 3, 12, 0, tzinfo=datetime.timezone.utc)]
                                                            Changelog Authors: ['meissner@suse.com', 'fcrozat@suse.com', 'dmueller@suse.com', 'coolo@suse.com', 'mvyskocil@suse.cz', 'coolo@suse.com', 'jengelh@medozas.de', 'ro@suse.de', 'uli@suse.com', 'adrian@suse.de', 'jengelh@medozas.de', 'jengelh@medozas.de', 'jengelh@medozas.de', 'jansimon.moeller@opensuse.org', 'crrodriguez@suse.de']
                                                              Changelog Texts: ['- libaio-optflags.diff: readd -stdlib to allow -fstack-protector-strong\n  builds (unclear why it was not allowed)\n- 01_link_libgcc.patch, 02_libdevdir.patch: refreshed', '- Add obsoletes/provides to baselibs.conf (bsc#881698)', '- Add libaio-aarch64-support.diff:\n  * add support for aarch64\n- Add libaio-generic-arch.diff:\n  * support all archtes (also aarch64)', '- fix baselibs.conf after shlib split', '- fix typo versoin/version', '- patch license to follow spdx.org standard', '- Remove redundant/unwanted tags/section (cf. specfile guidelines)\n- Employ shlib packaging', '- fix lib64 platform check', '- cross-build fix: use %__cc macro', '- drop debian arm hack to fix build on arm ;)', '- Update to libaio 0.3.109\n  * add ARM architecture support (grabbed from Debian arches tree)\n  * replace check of __i386__ with __LP64__ in test harness\n- refreshed patches', '- fix more symbolic links to not include a /usr/src/ prefix', '- update to libaio 0.3.107\n- add more patches from Debian to fix compile errors on SPARC\n- package baselibs.conf', '- add ARM support to libaio sources', '- remove static libraries\n- fix -devel package dependencies']
                                    Post-installation Interpreter + Arguments: /sbin/ldconfig
                                  Post-uninstallation Interpreter + Arguments: /sbin/ldconfig
                                                               Obsolete Names: ['libaio']
                                                   Cookie For Build Operation: sheep53 1527271964
                                                                 File Devices: [1, 1, 1, 1, 1]
                                                  Abstract File Inode Numbers: [1, 2, 3, 4, 5]
                                                               File Languages: ['', '', '', '', '']
                                                       Provided Names (Flags): [<DependencyFlags.EQUAL: 8>, <DependencyFlags.FIND_PROVIDES: 32768>, <DependencyFlags.FIND_PROVIDES: 32768>, <DependencyFlags.FIND_PROVIDES: 32768>, <DependencyFlags.EQUAL: 8>, <DependencyFlags.EQUAL: 8>]
                                                    Provided Names (Versions): ['0.3.109-1.25', '', '', '', '0.3.109-1.25', '0.3.109-1.25']
                                                       Obsolete Names (Flags): <DependencyFlags.LESS: 2>
                                                    Obsolete Names (Versions): ['0.3.109-1.25']
                                     Index Into Directory Names For Basenames: [0, 0, 1, 2, 2]
                                                                    Basenames: ['libaio.so.1', 'libaio.so.1.0.1', 'libaio1', 'COPYING', 'TODO']
                                                              Directory Names: ['/lib64/', '/usr/share/doc/packages/', '/usr/share/doc/packages/libaio1/']
                                               %{{optflags}} Value During Build: -fmessage-length=0 -grecord-gcc-switches -O2 -Wall -D_FORTIFY_SOURCE=2 -fstack-protector-strong -funwind-tables -fasynchronous-unwind-tables -fstack-clash-protection -g
                                                    Distribution-specific URL: obs://build.suse.de/SUSE:SLE-15:GA/standard/77c22af2cb0aefbc84316daac8f5b8ac-libaio
                                                               Payload Format: cpio
                                                      Payload Compressor Name: xz
                                                     Payload Compressor Level: 5
                                                             Package Platform: x86_64-suse-linux
                                                  Index Into Class Dictionary: [0, 1, 2, 3, 3]
                               Class Dictionary (File Class libmagic Entries): ['', 'ELF 64-bit LSB shared object, x86-64, version 1 (SYSV), dynamically linked, BuildID[sha1]=98eb213e51c843c9263a66ad0a5718c06a388a1c, stripped', 'directory', 'ASCII text']
Index Into File Dependencies Dictionary Denoting Start Of File's Dependencies: [0, 0, 0, 0, 0]
                       Number Of File Dependencies in Dependencies Dictionary: [0, 6, 0, 0, 0]
                                                 File Dependencies Dictionary: [1342177282, 1342177283, 1342177283, 1342177281, 1375731715, 1375731714]
                                                    Source Package Identifier: b'\xcc\x8cLq\xe6\xf8\xcb\xbf\xfa\x14\xcc6Li\x18\x85'
                                                        File Digest Algorithm: <FileDigestAlgorithm.SHA512: 8>
                                                  Header String Data Encoding: utf-8
                                   Cryptographic Digest Of Compressed Payload: ['f001b298e5add90e4d2fdbc442b92b8aae7beb5fac77159a5baa33fb7217d48a']
                                                     Payload Digest Algorithm: <FileDigestAlgorithm.SHA512: 8>
"""[1:-1].format(file_flags=file_flags, file_verification_flags=file_verification_flags, required_names_flags=required_names_flags),  # noqa: E501
            results,
        )

    def test_source(self) -> None:
        if sys.version_info < (3, 11):
            file_flags = "[<FileFlags.0: 0>, <FileFlags.0: 0>, <FileFlags.0: 0>, <FileFlags.0: 0>, <FileFlags.0: 0>, <FileFlags.0: 0>, <FileFlags.0: 0>, <FileFlags.0: 0>, <FileFlags.0: 0>, <FileFlags.0: 0>, <FileFlags.SPECFILE: 32>]"  # noqa: E501
            required_names_flags = "[<DependencyFlags.RPMLIB|EQUAL|LESS: 16777226>, <DependencyFlags.RPMLIB|EQUAL|LESS: 16777226>]"
            file_verification_flags = "[<VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>, <VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>, <VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>, <VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>, <VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>, <VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>, <VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>, <VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>, <VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>, <VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>, <VerifyFlags.2147483648|1073741824|536870912|268435456|134217728|67108864|33554432|16777216|8388608|4194304|2097152|1048576|524288|262144|131072|65536|32768|16384|8192|4096|2048|1024|512|CAPS|RDEV|MODE|MTIME|GROUP|USER|LINK_TO|SIZE|MD5: 4294967295>]"  # noqa: E501
        else:
            file_flags = "[<FileFlags: 0>, <FileFlags: 0>, <FileFlags: 0>, <FileFlags: 0>, <FileFlags: 0>, <FileFlags: 0>, <FileFlags: 0>, <FileFlags: 0>, <FileFlags: 0>, <FileFlags: 0>, <FileFlags.SPECFILE: 32>]"  # noqa: E501
            required_names_flags = "[<DependencyFlags.LESS|EQUAL|RPMLIB: 16777226>, <DependencyFlags.LESS|EQUAL|RPMLIB: 16777226>]"
            file_verification_flags = "[<VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>, <VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>, <VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>, <VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>, <VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>, <VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>, <VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>, <VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>, <VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>, <VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>, <VerifyFlags.MD5|SIZE|LINK_TO|USER|GROUP|MTIME|MODE|RDEV|CAPS|4294966784: 4294967295>]"  # noqa: E501

        with get_from_url(LIBAIO1__0_3_109_1_25__SRC_RPM) as rpm_path:
            results = rpm_tools.check_rpm_headers(rpm_path)
        self.assertEqual(
            r"""
                         Header Signatures: b'\x00\x00\x00>\x00\x00\x00\x07\xff\xff\xffp\x00\x00\x00\x10'
           OpenPGP RSA Signature Of Header: b'\x89\x01\x15\x03\x05\x00[\x08R$p\xaf\x9e\x819\xdb|\x82\x01\x08H\x05\x08\x00\x85wx\x9dG\xe0$\xfe\xf8\xf1*y\xfa\xc1\x0c\xa1-\xc9 B\xed\xf3\xfa\'\x80v\x91\xe1Wl\x02\xaf\x94\x06>^$\xe1\xca\xf3\x06\xc5\xaa\xa6\xa1\x85\x98\xa3\xdc\x00+\x03\xdd\x98\xef\xd2\x1d\x9b*\xb6Bu6J\xad\xf0\x12\xc9\xbb8"\x84\x9e\xbd\x87)\x94\xf3\xb4\x832\x99\xc1k\xf7r\xa3\xbf\xe9\xecA\xaf\x86\x02\xee\xb7\xb36;8\xc5\x19{{C\xfat\xdc\xec,\xdc\xef)\x92\xc4\xd3\xcd7\x88\xe0q\x18@\x95\x89n\xc3\x8a\xaah\\\x83\xd4L\xaa.\x00\xf3[\xce\xab\x17*D\xe4\xa4-\x1d-\x89\x15^\x02m\xec"\xca\n\xc4\xb9>\xcbS?\x14\x03\x00\x05#x\xb5\x83f\xc68A\x16\xe2k\x0e\x7f\x9bi\x9a\xe9\x88\xda\xf2j\t!K\xe7d\xa5\xf3\xaa2\xaay\xf2v\'\xb0\x11\xc9\x9an\xf2\x06\xefW6d\xa8\xf1\x85\xc7)\xc1\x05<\xea|\x8cJ\xb2\xc0\xd9\xe7\x10V=\xcd4\xf5:\xff\xf1\x1cL\xe5\xff9_\x14\'\x85u\xcd\xae\x8f\x11J\xf8\xd46'
                     SHA1 Digest Of Header: 42dd9660c0752a72f332b198f09c57a2a27a621e
                   SHA256 Digest Of Header: 453f75a56cd0fec8359b87d82e5423d332a4fe4c4fd2379261eea0d784e07a9a
                    Installed Package Size: 158737
       PGP Signature From Signature Header: b'\x89\x01\x15\x03\x05\x00[\x08R$p\xaf\x9e\x819\xdb|\x82\x01\x08\xc5c\x08\x00\xa5a\xe4\x8f\xa4_R\xb1s;:,+He40\x13\xdaZ\xb9\x83\xbf\xb4\xc7\xa2\xe1\x9c\x80\xcf\x07\x02\x04\xbd0\x84\xfa\xf5\x93\xaf\xb0\xf3Y\xe7C\x84\xc1wFe\'\xc3\xcaGC\xb8}}\x8d\xd4E\x11\x03\xf1\xab\xe8\xf8\x9aw\xcb\x8e\xf7J(\x90\xb8h\xe0\x07\xf3\xbd\x12\x0b\xac\xa0\xf0#\x1c"/DRL\xfep5\x11@\x07cS?\xfb\\)&\xc86\x18\xb1G\xdf\xcd\xe7\xc1(1S-\xdc]\xc1\x00\xd6\x9a\x85D\xf1\xb6\x9f\xc2\xc2\xa8o\xe8\x9d\xceGk\xca\xdb.Id0[ZW\x1f\xb1C\xdd5\x82\x83\x1a\xa0\xca\x89\x18G\xfd\x98Y\x1d.D\x8b\xbdJuq\x13\xaf\xb1L\xd13\xcer6}}(\xd9\x7fhb\x97\x8a\x0b\x859\xf5(3\xd9\x0b\xcf=N\x05Ljh\x11^\xe0\x8f\x1b\xb4\x03\xb6\x1aq\xf0Z\xcd\xf6nf+\x86\x18\xa5zD\xcf\xab\x83\x16\x8e\xbceF<\xa8\x9b\x86\xf5\xd6\xf51G^\\\xad\x9d\x87\xb5\x12\x7f\x94\'v6n\x98'
            MD5 Digest Of Header + Payload: b'\xcc\x8cLq\xe6\xf8\xcb\xbf\xfa\x14\xcc6Li\x18\x85'
                              Payload Size: 160320
                            Reserved Space: 4128
         Unmodified, Original Header Image: b'\x00\x00\x00?\x00\x00\x00\x07\xff\xff\xfc\xb0\x00\x00\x00\x10'
                Header Translation Locales: ['C']
                              Package Name: libaio
                           Package Version: 0.3.109
                           Package Release: 1.25
                          One-line Summary: Linux-Native Asynchronous I/O Access Library
                    Multi-line Description: The Linux-native asynchronous I/O facility ("async I/O", or "aio") has
a richer API and capability set than the simple POSIX async I/O
facility. This library provides the Linux-native API for async I/O. The
POSIX async I/O facility requires this library to provide
kernel-accelerated async I/O capabilities, as do applications that
require the Linux-native async I/O API.
                        Package Build Time: datetime.datetime(2018, 5, 25, 18, 12, 44, tzinfo=datetime.timezone.utc)
                  Hostname Of Build System: sheep53
                         Distribution Name: SUSE Linux Enterprise 15
                            Package Vendor: SUSE LLC <https://www.suse.com/>
                       License Of Contents: LGPL-2.1+
                                  Packager: https://www.suse.com/
                             Package Group: Development/Libraries/C and C++
                         Source File Names: ['baselibs.conf', 'libaio-0.3.109.tar.bz2']
                          Patch File Names: ['libaio-generic-arch.diff', 'libaio-aarch64-support.diff', '03_man_errors.patch', '02_libdevdir.patch', '01_link_libgcc.patch', '00_arches_sh.patch', '00_arches.patch', 'libaio-optflags.diff']
                              Upstream URL: http://kernel.org/pub/linux/libs/aio/
                          Operating System: linux
                              Architecture: x86_64
              File Sizes (when all < 4 GB): [22666, 5028, 500, 2159, 73868, 207, 43579, 1018, 2053, 512, 7147]
                          Unix Files Modes: ['<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|READ_BY_OWNER|WRITE_BY_OWNER|READ_BY_GROUP|READ_BY_OTHERS: 33188>', '<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|READ_BY_OWNER|WRITE_BY_OWNER|READ_BY_GROUP|READ_BY_OTHERS: 33188>', '<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|READ_BY_OWNER|WRITE_BY_OWNER|READ_BY_GROUP|READ_BY_OTHERS: 33188>', '<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|READ_BY_OWNER|WRITE_BY_OWNER|READ_BY_GROUP|READ_BY_OTHERS: 33188>', '<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|READ_BY_OWNER|WRITE_BY_OWNER|READ_BY_GROUP|READ_BY_OTHERS: 33188>', '<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|READ_BY_OWNER|WRITE_BY_OWNER|READ_BY_GROUP|READ_BY_OTHERS: 33188>', '<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|READ_BY_OWNER|WRITE_BY_OWNER|READ_BY_GROUP|READ_BY_OTHERS: 33188>', '<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|READ_BY_OWNER|WRITE_BY_OWNER|READ_BY_GROUP|READ_BY_OTHERS: 33188>', '<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|READ_BY_OWNER|WRITE_BY_OWNER|READ_BY_GROUP|READ_BY_OTHERS: 33188>', '<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|READ_BY_OWNER|WRITE_BY_OWNER|READ_BY_GROUP|READ_BY_OTHERS: 33188>', '<FileModes.IS_REGULAR_FILE|READ_BY_OWNER_ALTERNATIVE|WRITE_BY_OWNER_ALTERNATIVE|READ_BY_OWNER|WRITE_BY_OWNER|READ_BY_GROUP|READ_BY_OTHERS: 33188>']
              Device IDs (of device files): [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
              File Modification Timestamps: [datetime.datetime(2011, 3, 24, 8, 43, 41, tzinfo=datetime.timezone.utc), datetime.datetime(2011, 3, 24, 8, 43, 41, tzinfo=datetime.timezone.utc), datetime.datetime(2016, 4, 22, 14, 18, tzinfo=datetime.timezone.utc), datetime.datetime(2016, 4, 22, 14, 18, tzinfo=datetime.timezone.utc), datetime.datetime(2010, 2, 11, 19, 16, 55, tzinfo=datetime.timezone.utc), datetime.datetime(2014, 8, 26, 11, 53, 35, tzinfo=datetime.timezone.utc), datetime.datetime(2009, 11, 20, 14, 31, 19, tzinfo=datetime.timezone.utc), datetime.datetime(2013, 3, 1, 10, 32, 38, tzinfo=datetime.timezone.utc), datetime.datetime(2013, 3, 1, 10, 32, 38, tzinfo=datetime.timezone.utc), datetime.datetime(2016, 4, 22, 14, 18, tzinfo=datetime.timezone.utc), datetime.datetime(2018, 5, 25, 18, 12, 42, tzinfo=datetime.timezone.utc)]
                                 File Hash: ['ab4b71ed9914e09c9470ed193e6049ab251e2f13bc17e7a54f5c74c0965c841e', 'e14215abee7b135f9d0d832e3544b30dc3e7540b0ccec8414ce42b1a47a86986', '9e615a1fe27800756e68017d25c8e029614a72993d27d4ecfe59d410867a0b8f', '2474d6f7378e254d47370b68a992f08ad30e8b522234c283754361accec55aea', 'cb4b2b1a128f9eb1ab6f39660359098950cb1906eb883d8b6ad38e8fa8c319fa', 'aa4d278101271a5a269ca3471402203d70f8da05c42d46c52be1b2e24f95cf59', 'b5cefce0a3cb49f8dca4d00e9480c0d9b45b75863bd44764156e322ee214e794', '76beebbc9cc994a1be1dbe686a4c20d5374462da03ffa512083ccd2dac940c1e', 'e91be00e676f63a54a82699c3e20766248587edc09f84bfd8207743519a8dfca', 'fa8847f11938773bd93ab17bd2b1319bb2fe094561aeaaf75fd840205ff112ff', '6261948d71f6ff1c4c321ddef3bb655196d898ac08b44ec7ec1ee20d9d59dcf6']
                      File Symlink Targets: ['', '', '', '', '', '', '', '', '', '', '']
                                File Flags: {file_flags}
                     File Unix Owner Names: ['root', 'root', 'root', 'root', 'root', 'root', 'root', 'root', 'root', 'root', 'root']
                     File Unix Group Names: ['root', 'root', 'root', 'root', 'root', 'root', 'root', 'root', 'root', 'root', 'root']
                   File Verification Flags: {file_verification_flags}
                    Required Names (Flags): {required_names_flags}
                            Required Names: ['rpmlib(CompressedFileNames)', 'rpmlib(FileDigests)']
                 Required Names (Versions): ['3.0.4-1', '4.6.0-1']
                  RPM Version For Building: 4.14.1
                      Changelog Timestamps: [datetime.datetime(2016, 4, 17, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2014, 8, 26, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2013, 3, 1, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2012, 2, 17, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2012, 2, 16, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2012, 2, 13, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2011, 11, 28, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2011, 11, 28, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2011, 10, 5, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2011, 9, 30, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2011, 3, 15, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2010, 2, 12, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2010, 1, 23, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2009, 8, 2, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2009, 3, 3, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2008, 12, 10, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2008, 12, 4, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2008, 4, 10, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2007, 9, 27, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2007, 8, 2, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2006, 1, 25, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2005, 5, 5, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2005, 4, 27, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2004, 12, 1, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2004, 4, 20, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2004, 3, 12, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2004, 3, 2, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2004, 1, 11, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2003, 10, 1, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2003, 4, 23, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2003, 4, 23, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2003, 4, 11, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2003, 4, 3, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2002, 10, 1, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2002, 9, 20, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2002, 9, 19, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2002, 9, 17, 12, 0, tzinfo=datetime.timezone.utc), datetime.datetime(2002, 9, 4, 12, 0, tzinfo=datetime.timezone.utc)]
                         Changelog Authors: ['meissner@suse.com', 'fcrozat@suse.com', 'dmueller@suse.com', 'coolo@suse.com', 'mvyskocil@suse.cz', 'coolo@suse.com', 'jengelh@medozas.de', 'ro@suse.de', 'uli@suse.com', 'adrian@suse.de', 'jengelh@medozas.de', 'jengelh@medozas.de', 'jengelh@medozas.de', 'jansimon.moeller@opensuse.org', 'crrodriguez@suse.de', 'olh@suse.de', 'olh@suse.de', 'ro@suse.de', 'hare@suse.de', 'hare@suse.de', 'mls@suse.de', 'schwab@suse.de', 'kukuk@suse.de', 'kukuk@suse.de', 'meissner@suse.de', 'kukuk@suse.de', 'ro@suse.de', 'adrian@suse.de', 'schwab@suse.de', 'coolo@suse.de', 'coolo@suse.de', 'ro@suse.de', 'kukuk@suse.de', 'meissner@suse.de', 'fehr@suse.de', 'schwab@suse.de', 'ro@suse.de', 'fehr@suse.de']
                           Changelog Texts: ['- libaio-optflags.diff: readd -stdlib to allow -fstack-protector-strong\n  builds (unclear why it was not allowed)\n- 01_link_libgcc.patch, 02_libdevdir.patch: refreshed', '- Add obsoletes/provides to baselibs.conf (bsc#881698)', '- Add libaio-aarch64-support.diff:\n  * add support for aarch64\n- Add libaio-generic-arch.diff:\n  * support all archtes (also aarch64)', '- fix baselibs.conf after shlib split', '- fix typo versoin/version', '- patch license to follow spdx.org standard', '- Remove redundant/unwanted tags/section (cf. specfile guidelines)\n- Employ shlib packaging', '- fix lib64 platform check', '- cross-build fix: use %__cc macro', '- drop debian arm hack to fix build on arm ;)', '- Update to libaio 0.3.109\n  * add ARM architecture support (grabbed from Debian arches tree)\n  * replace check of __i386__ with __LP64__ in test harness\n- refreshed patches', '- fix more symbolic links to not include a /usr/src/ prefix', '- update to libaio 0.3.107\n- add more patches from Debian to fix compile errors on SPARC\n- package baselibs.conf', '- add ARM support to libaio sources', '- remove static libraries\n- fix -devel package dependencies', '- use Obsoletes: -XXbit only for ppc64 to help solver during distupgrade\n  (bnc#437293)', '- obsolete old -XXbit packages (bnc#437293)', '- added baselibs.conf file to build xxbit packages\n  for multilib support', '- Fix dangling symlink (#307063)', '- Use RPM_OPT_FLAGS\n- Fix installation directories', '- converted neededforbuild to BuildRequires', '- Fix ia64 assembler.', '- Update to version 0.3.104', '- Update to version 0.3.102 [#44374]', '- fixed ppc64 alignment problems. [#38801/LTC#7503]', '- Update to 0.3.98 [Bug #35266]', '- use -fPIC for shared objects on ppc', '- add %defattr and %run_ldconfig', '- Fix for ia64.', '- fix build for lib64', '- use BuildRoot', '- fix header to be includable with glibc (#26033)', '- Add missing "const" to libaio.h [#26030]', '- Fixed __syscall_return for ppc.', '- Add syscall defines for x86_64\n- add Andreas fix for testsuite main program to compile on x86_64\n- add another fix to make testsuite build again on ia64', '- Add missing bits for ia64.', '- removed bogus self-provides', '- make package from  libaio-0.3.15-2.5']
                Cookie For Build Operation: sheep53 1527271964
                              File Devices: [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]
               Abstract File Inode Numbers: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
                            File Languages: ['', '', '', '', '', '', '', '', '', '', '']
                                Source RPM: 1
  Index Into Directory Names For Basenames: [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
                                 Basenames: ['00_arches.patch', '00_arches_sh.patch', '01_link_libgcc.patch', '02_libdevdir.patch', '03_man_errors.patch', 'baselibs.conf', 'libaio-0.3.109.tar.bz2', 'libaio-aarch64-support.diff', 'libaio-generic-arch.diff', 'libaio-optflags.diff', 'libaio.spec']
                           Directory Names: ['']
                 Distribution-specific URL: obs://build.suse.de/SUSE:SLE-15:GA/standard/77c22af2cb0aefbc84316daac8f5b8ac-libaio
                            Payload Format: cpio
                   Payload Compressor Name: gzip
                  Payload Compressor Level: 9
                     File Digest Algorithm: <FileDigestAlgorithm.SHA512: 8>
               Header String Data Encoding: utf-8
Cryptographic Digest Of Compressed Payload: ['75c03228af86b5aa125966962a19ec402ab330054264f74a4d877dd357a46c26']
                  Payload Digest Algorithm: <FileDigestAlgorithm.SHA512: 8>
"""[1:-1].format(file_flags=file_flags, file_verification_flags=file_verification_flags, required_names_flags=required_names_flags),  # noqa: E501
            results,
        )



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

from __future__ import annotations

import shutil
from collections.abc import Generator
from pathlib import Path
from tempfile import mkdtemp
from types import TracebackType
from typing import cast

from typecode import magic2  # type: ignore[import-untyped]


def get_files_from_directory(
    directory: str | Path,
    prefix: str | None = None,
) -> Generator[tuple[Path, str]]:
    """
    Get the files from the given directory, recursively.

    :param directory: The directory to walk through.
    :param prefix: Custom prefix to use.
    :return: For each file, the complete Path object as well as the path string
             relative to the given directory.
    """
    directory_string = str(directory) if prefix is None else prefix
    common_prefix_length = len(directory_string) + int(
        not directory_string.endswith("/"),
    )

    for path in sorted(Path(directory).rglob("*"), key=str):
        if path.is_dir():
            continue
        short_path = str(path)[common_prefix_length:]
        yield path, short_path


class DirectoryWithFixedNameContext:
    """
    Create the directory with the given name. Similar to `tempfile.TemporaryDirectory`, but with a fixed name.
    """

    def __init__(
            self,
            directory: str | Path,
            fallback_to_random_if_exists: bool = True,
            delete_afterwards: bool = True,
    ) -> None:
        """
        :param directory: The target directory to create temporarily.
        :param fallback_to_random_if_exists: Specify whether to fail if the target directory
                                             already exists or whether to fall back to a
                                             random directory with the same parent in this case.
        :param delete_afterwards: Specify whether to delete the directory on exit.
        """
        self.directory = Path(directory)
        self._consider_fallback = fallback_to_random_if_exists
        self._delete_afterwards = delete_afterwards

    def __enter__(self) -> Path:
        if self.directory.exists():
            if not self._consider_fallback:
                # Let `pathlib` handle the error reporting which contains more details.
                self.directory.mkdir(parents=False, exist_ok=False)
            else:
                self.directory = Path(mkdtemp(dir=self.directory.parent))
        else:
            self.directory.mkdir(parents=False, exist_ok=False)
        return self.directory

    def __exit__(self, type_: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None) -> bool | None:
        if self._delete_afterwards:
            shutil.rmtree(self.directory)

        if type_:
            # Error.
            return None
        return True


def get_mime_type(path: Path) -> str:
    """
    Get the mime type.

    :param: The file to check.
    :return: The guessed mime type.
    """
    return cast(str, magic2.mime_type(path))


def get_file_type(path: Path) -> str:
    """
    Get the file type.

    :param: The file to check.
    :return: The guessed file type.
    """
    return cast(str, magic2.file_type(path))



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

from __future__ import annotations

import re
import sys
from pathlib import Path
from tempfile import TemporaryDirectory
from unittest import TestCase

from license_tools.utils.path_utils import DirectoryWithFixedNameContext, get_file_type, get_files_from_directory, get_mime_type
from tests import get_file
from tests.data import LICENSE_PATH, SETUP_PATH


class GetFilesFromDirectoryTestCase(TestCase):
    def test_get_files_from_directory(self) -> None:
        with TemporaryDirectory() as temporary_directory:
            directory = Path(temporary_directory)

            directory.joinpath("module1.py").touch()
            directory.joinpath("module2.py").touch()
            directory.joinpath("submodule").mkdir(parents=True)
            directory.joinpath("submodule").joinpath("nested.py").touch()
            directory.joinpath("empty").joinpath("sub").mkdir(parents=True)
            directory.joinpath("empty").joinpath("sub").joinpath("hello.py").touch()

            result = list(get_files_from_directory(temporary_directory))
            self.assertListEqual(
                [
                    (directory / "empty" / "sub" / "hello.py", "empty/sub/hello.py"),
                    (directory / "module1.py", "module1.py"),
                    (directory / "module2.py", "module2.py"),
                    (directory / "submodule" / "nested.py", "submodule/nested.py"),
                ],
                result,
            )


class DirectoryWithFixedNameContextTestCase(TestCase):
    def test_normal(self) -> None:
        with TemporaryDirectory() as tempdir:
            with DirectoryWithFixedNameContext(Path(tempdir) / "foo") as target:
                self.assertTrue(target.is_dir())
            self.assertFalse(target.is_dir())

    def test_missing_parent(self) -> None:
        with TemporaryDirectory() as tempdir:
            target_name = Path(tempdir) / "foo" / "bar"
            with self.assertRaisesRegex(
                    expected_exception=FileNotFoundError,
                    expected_regex=fr"^\[Errno 2\] No such file or directory: '{re.escape(str(target_name))}'$",
            ):
                with DirectoryWithFixedNameContext(Path(tempdir) / "foo" / "bar") as target:
                    self.assertTrue(target.is_dir())

    def test_already_existing(self) -> None:
        with TemporaryDirectory() as tempdir:
            target_name = Path(tempdir) / "foo"
            target_name.mkdir()
            with self.assertRaisesRegex(
                    expected_exception=FileExistsError,
                    expected_regex=fr"^\[Errno 17\] File exists: '{re.escape(str(target_name))}'$",
            ):
                with DirectoryWithFixedNameContext(Path(tempdir) / "foo", fallback_to_random_if_exists=False) as target:
                    self.assertTrue(target.is_dir())

        with TemporaryDirectory() as tempdir:
            target_name = Path(tempdir) / "foo"
            target_name.mkdir()
            with DirectoryWithFixedNameContext(Path(tempdir) / "foo", fallback_to_random_if_exists=True) as target:
                self.assertTrue(target.is_dir())
                self.assertNotEqual(target_name, target)

    def test_deleted_in_between(self) -> None:
        with TemporaryDirectory() as tempdir:
            target_name = Path(tempdir) / "foo"
            if sys.version_info < (3, 12):
                expected_regex = fr"^\[Errno 2\] No such file or directory: '{re.escape(str(target_name))}'$"
            else:
                expected_regex = fr"^\[Errno 2\] No such file or directory: {re.escape(repr(target_name))}$"
            with self.assertRaisesRegex(
                    expected_exception=FileNotFoundError,
                    expected_regex=expected_regex,
            ):
                with DirectoryWithFixedNameContext(Path(tempdir) / "foo") as target:
                    self.assertTrue(target.is_dir())
                    target.rmdir()

    def test_do_not_delete_afterwards(self) -> None:
        with TemporaryDirectory() as tempdir:
            with DirectoryWithFixedNameContext(Path(tempdir) / "foo", delete_afterwards=False) as target:
                self.assertTrue(target.is_dir())
            self.assertTrue(target.is_dir())


class GetMimeTypeTestCase(TestCase):
    def test_get_mime_type(self) -> None:
        self.assertEqual(
            "text/x-script.python",
            get_mime_type(SETUP_PATH),
        )
        self.assertEqual(
            "text/plain",
            get_mime_type(LICENSE_PATH),
        )

        with get_file("croissant.jpg") as path:
            self.assertEqual(
                "image/jpeg",
                get_mime_type(path),
            )


class GetFileTypeTestCase(TestCase):
    def test_get_file_type(self) -> None:
        self.assertEqual(
            "Python script, ASCII text executable",
            get_file_type(SETUP_PATH),
        )
        self.assertEqual(
            "ASCII text",
            get_file_type(LICENSE_PATH),
        )

        with get_file("croissant.jpg") as path:
            self.assertEqual(
                "JPEG image data, Exif standard: [TIFF image data, little-endian, direntries=0], baseline, precision 8, 1280x853, components 3",
                get_file_type(path),
            )

        with get_file("Carlito-Regular.ttf") as path:
            self.assertEqual(
                (
                    'TrueType Font data, 17 tables, 1st "GDEF", 15 names, Microsoft, language 0x409, '
                    'Copyright 2013 The Carlito Project Authors (https://github.com/googlefonts/carlito)CarlitoRegula'
                ),
                get_file_type(path),
            )



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

from __future__ import annotations

import os
import subprocess
import sys
import time
from pathlib import Path
from tempfile import TemporaryDirectory
from unittest import TestCase

from license_tools.__main__ import log_level_type
from license_tools.utils.path_utils import get_files_from_directory
from tests.data import TYPING_EXTENSION_4_8_0__EXPECTED_OUTPUT

EXAMPLE_CARGO_LOCK_FILE = """
# This file is automatically @generated by Cargo.
# It is not intended for manual editing.
version = 3

[[package]]
name = "autocfg"
version = "1.1.0"
source = "registry+https://github.com/rust-lang/crates.io-index"
checksum = "d468802bab17cbc0cc575e9b053f41e72aa36bfa6b7f55e3529ffa43161b97fa"

[[package]]
name = "base64"
version = "0.21.7"
source = "registry+https://github.com/rust-lang/crates.io-index"
checksum = "9d297deb1925b89f2ccc13d7635fa0714f12c87adce1c75356b39ca9b7178567"

[[package]]
name = "bitflags"
version = "1.3.2"
source = "registry+https://github.com/rust-lang/crates.io-index"
checksum = "bef38d45163c2f1dde094a7dfd33ccf595c92905c8f8f4fdc18d06fb1037718a"
"""

EXAMPLE_CARGO_LOCK_FILE__ONE_PACKAGE_ONLY = "".join(EXAMPLE_CARGO_LOCK_FILE.splitlines(keepends=True)[:10])


class LogLevelTypeTestCase(TestCase):
    def test_constant_name(self) -> None:
        self.assertEqual(40, log_level_type("error"))
        self.assertEqual(40, log_level_type("ERROR"))

    def test_integer(self) -> None:
        self.assertEqual(40, log_level_type(40))
        self.assertEqual(42, log_level_type(42))

    def test_invalid(self) -> None:
        with self.assertRaisesRegex(
                expected_exception=AttributeError, expected_regex=r"^module 'logging' has no attribute 'ABC'$",
        ):
            log_level_type("abc")


class MainTestCase(TestCase):
    @property
    def custom_env(self) -> dict[str, str]:
        custom_env = os.environ.copy()
        custom_env["COLUMNS"] = "100"
        custom_env["COVERAGE_PROCESS_START"] = "pyproject.toml"
        return custom_env

    def test_retrieval(self) -> None:
        # The tests started to fail with Python 3.13 when using parallel processing within the subprocess here,
        # while later on they became very slow on Python 3.9 when Python 3.13 worked normally.
        # Trying to debug this did not yet yield any useful insights on where the issue actually is.
        # As this should not hurt real-world usage (nobody should really be required to run this in a subprocess
        # outside the tests), just use the slower sequential analysis here.
        result = subprocess.run(
            [sys.executable, "-m", "license_tools", "--package", "typing_extensions==4.8.0", "--index-url", "https://pypi.org/simple", "--jobs", "1"],
            check=False, capture_output=True, env=self.custom_env,
        )
        self.assertEqual(0, result.returncode, result)
        self.assertEqual(b"", result.stderr)
        self.assertEqual(TYPING_EXTENSION_4_8_0__EXPECTED_OUTPUT, result.stdout.decode("UTF-8"))

    def test_cargo_lock_download(self) -> None:
        time.sleep(1)
        with TemporaryDirectory() as target_directory, TemporaryDirectory() as source_directory:
            cargo_lock = Path(source_directory) / "Cargo.lock"
            cargo_lock.write_text(EXAMPLE_CARGO_LOCK_FILE)

            result = subprocess.run(
                [sys.executable, "-m", "license_tools", "--cargo-lock-download", "--cargo-lock", cargo_lock, "--target-directory", target_directory],
                check=False, capture_output=True, env=self.custom_env,
            )
            self.assertEqual(0, result.returncode, result)
            self.assertEqual(b"", result.stderr)
            self.assertEqual(b"", result.stdout)

            actual = [x[1] for x in get_files_from_directory(target_directory)]
            self.assertEqual(
                [
                    "autocfg_1.1.0.crate",
                    "base64_0.21.7.crate",
                    "bitflags_1.3.2.crate",
                ],
                actual,
            )

    def test_logging(self) -> None:
        time.sleep(1)
        with TemporaryDirectory() as target_directory, TemporaryDirectory() as source_directory:
            cargo_lock = Path(source_directory) / "Cargo.lock"
            cargo_lock.write_text(EXAMPLE_CARGO_LOCK_FILE__ONE_PACKAGE_ONLY)

            result = subprocess.run(
                [
                    sys.executable, "-m", "license_tools", "--cargo-lock-download", "--cargo-lock", cargo_lock, "--target-directory", target_directory,
                    "--log-level", "INFO",
                ],
                check=False, capture_output=True, env=self.custom_env,
            )
            self.assertEqual(0, result.returncode, result)
            self.assertEqual(
                (
                    f"INFO:license_tools.utils.download_utils:Downloading https://crates.io/api/v1/crates/autocfg/1.1.0/download to "
                    f"{target_directory}/autocfg_1.1.0.crate ...\n"
                ),
                result.stderr.decode("UTF-8"),
            )
            self.assertEqual(b"", result.stdout)

            actual = [x[1] for x in get_files_from_directory(target_directory)]
            self.assertEqual(["autocfg_1.1.0.crate"], actual)



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

from __future__ import annotations

import datetime
from pathlib import Path
from tempfile import NamedTemporaryFile, mkdtemp
from typing import cast
from unittest import TestCase

from faker import Faker

from license_tools.retrieval import RetrievalFlags
from license_tools.tools import scancode_tools
from license_tools.tools.scancode_tools import (
    Copyright,
    Copyrights,
    Emails,
    FileInfo,
    FileResults,
    Holder,
    LicenseDetection,
    LicenseMatch,
    Licenses,
    PackageResults,
    Party,
    Url,
    Urls,
)
from tests import get_from_url
from tests.data import LIBAIO1__0_3_109_1_25__RPM, LICENSE_PATH, SETUP_PATH, SETUP_PY_LICENSES


class LicensesTestCase(TestCase):
    def test_get_scores_of_detected_license_expression_spdx(self) -> None:
        result = SETUP_PY_LICENSES.get_scores_of_detected_license_expression_spdx()
        self.assertEqual([100.0, 100.0], result)

        file_results = FileResults(
            path=LICENSE_PATH, short_path="LICENSE.txt", retrieve_licenses=True,
        )
        licenses = cast(Licenses, file_results.licenses)
        result = licenses.get_scores_of_detected_license_expression_spdx()
        self.assertEqual([100.0], result)


class FileResultsTestCase(TestCase):
    def assert_not_requested(
        self, result: FileResults, fields: list[str], invert: bool = False,
    ) -> None:
        method = self.assertIsNotNone if invert else self.assertIsNone
        for field in fields:
            with self.subTest(field=field):
                method(getattr(result, field))

    def test_full(self) -> None:
        flags = cast(dict[str, bool], RetrievalFlags.all(as_kwargs=True))
        del flags["retrieve_ldd_data"]
        del flags["retrieve_font_data"]
        del flags["retrieve_python_metadata"]
        del flags["retrieve_cargo_metadata"]
        del flags["retrieve_image_metadata"]
        self.assertEqual(4, len(flags))
        result = FileResults(
            path=SETUP_PATH, short_path="setup.py", retrieve_licenses=True,
            retrieve_copyrights=flags["retrieve_copyrights"],
            retrieve_emails=flags["retrieve_emails"],
            retrieve_urls=flags["retrieve_urls"],
            retrieve_file_info=flags["retrieve_file_info"],
        )
        self.assertEqual(SETUP_PATH, result.path)
        self.assertEqual("setup.py", result.short_path)
        self.assert_not_requested(
            result=result,
            fields=["copyrights", "emails", "urls", "licenses", "file_info"],
            invert=True,
        )

    def test_none(self) -> None:
        result = FileResults(path=Path("/tmp/dummy.py"), short_path="dummy.py")
        self.assertEqual(Path("/tmp/dummy.py"), result.path)
        self.assertEqual("dummy.py", result.short_path)
        self.assert_not_requested(
            result=result,
            fields=["copyrights", "emails", "urls", "licenses", "file_info"],
        )

    def test_retrieve_copyrights(self) -> None:
        result = scancode_tools.FileResults(
            path=SETUP_PATH, short_path="setup.py", retrieve_copyrights=True,
        )
        self.assertEqual(SETUP_PATH, result.path)
        self.assertEqual("setup.py", result.short_path)
        self.assert_not_requested(
            result=result, fields=["emails", "urls", "licenses", "file_info"],
        )
        expected = Copyrights(
            copyrights=[
                Copyright(
                    copyright="Copyright (c) stefan6419846", start_line=1, end_line=1,
                ),
            ],
            holders=[Holder(holder="stefan6419846", start_line=1, end_line=1)],
            authors=[],
        )
        self.assertEqual(expected, result.copyrights)

    def test_retrieve_copyrights__limit(self) -> None:
        with NamedTemporaryFile(mode="w+t") as source_file:
            fake = Faker()
            for _ in range(123):
                source_file.write(f"Copyright (c) {fake.year()} {fake.name()}\n")
            source_file.flush()
            source_file.seek(0)

            result = scancode_tools.FileResults(
                path=Path(source_file.name), short_path="dummy.txt", retrieve_copyrights=True,
            )
            self.assertIsNotNone(result.copyrights)
            self.assertEqual(123, len(cast(Copyrights, result.copyrights).copyrights), result.copyrights)

    def test_retrieve_emails(self) -> None:
        result = scancode_tools.FileResults(
            path=SETUP_PATH, short_path="setup.py", retrieve_emails=True,
        )
        self.assertEqual(SETUP_PATH, result.path)
        self.assertEqual("setup.py", result.short_path)
        self.assert_not_requested(
            result=result, fields=["copyrights", "urls", "licenses", "file_info"],
        )
        expected = Emails(emails=[])
        self.assertEqual(expected, result.emails)

    def test_retrieve_emails__limit(self) -> None:
        with NamedTemporaryFile(mode="w+t") as source_file:
            fake = Faker()
            for _ in range(123):
                # `fake.email()` does not work here, as `cluecode.finder.find_emails` will filter `example.org`
                # for example due to `cluecode.finder_data.JUNK*`.
                source_file.write(f"{fake.company_email()}\n")
            source_file.flush()
            source_file.seek(0)

            result = scancode_tools.FileResults(
                path=Path(source_file.name), short_path="dummy.txt", retrieve_emails=True,
            )
            self.assertIsNotNone(result.emails)
            self.assertEqual(50, len(cast(Emails, result.emails).emails), result.emails)

            result = scancode_tools.FileResults(
                path=Path(source_file.name), short_path="dummy.txt", retrieve_emails=True, email_limit=None,
            )
            self.assertIsNotNone(result.emails)
            self.assertEqual(123, len(cast(Emails, result.emails).emails), result.emails)

    def test_retrieve_urls(self) -> None:
        result = scancode_tools.FileResults(
            path=SETUP_PATH, short_path="setup.py", retrieve_urls=True,
        )
        self.assertEqual(SETUP_PATH, result.path)
        self.assertEqual("setup.py", result.short_path)
        self.assert_not_requested(
            result=result, fields=["copyrights", "emails", "licenses", "file_info"],
        )
        expected = Urls(
            urls=[
                Url(
                    url="http://www.apache.org/licenses/LICENSE-2.0",
                    start_line=3,
                    end_line=3,
                ),
                Url(
                    url="https://github.com/stefan6419846/license_tools",
                    start_line=21,
                    end_line=21,
                ),
            ],
        )
        self.assertEqual(expected, result.urls)

    def test_retrieve_urls__limit(self) -> None:
        with NamedTemporaryFile(mode="w+t") as source_file:
            fake = Faker()
            for i in range(123):
                # Without the index, the generated URLs tend to not be unique with 123 generated values.
                source_file.write(f"{fake.url()}{i}\n")
            source_file.flush()
            source_file.seek(0)

            result = scancode_tools.FileResults(
                path=Path(source_file.name), short_path="dummy.txt", retrieve_urls=True,
            )
            self.assertIsNotNone(result.urls)
            self.assertEqual(50, len(cast(Urls, result.urls).urls), result.urls)

            result = scancode_tools.FileResults(
                path=Path(source_file.name), short_path="dummy.txt", retrieve_urls=True, url_limit=None,
            )
            self.assertIsNotNone(result.urls)
            self.assertEqual(123, len(cast(Urls, result.urls).urls), result.urls)

    def test_retrieve_licenses(self) -> None:
        self.maxDiff = None
        result = scancode_tools.FileResults(
            path=SETUP_PATH, short_path="setup.py", retrieve_licenses=True,
        )
        self.assertEqual(SETUP_PATH, result.path)
        self.assertEqual("setup.py", result.short_path)
        self.assert_not_requested(
            result=result, fields=["copyrights", "emails", "urls", "file_info"],
        )
        self.assertEqual(SETUP_PY_LICENSES, result.licenses)

    def test_retrieve_file_info(self) -> None:
        with NamedTemporaryFile(suffix=".py") as file_object:
            path = Path(file_object.name)
            path.write_text('print("Hello World!")\n')
            result = scancode_tools.FileResults(
                path=path, short_path="test.py", retrieve_file_info=True,
            )

        self.assertEqual(path, result.path)
        self.assertEqual("test.py", result.short_path)
        self.assert_not_requested(
            result=result, fields=["copyrights", "emails", "urls", "licenses"],
        )
        expected = FileInfo(
            date=datetime.date.today(),
            size=22,
            sha1="e343a35cf2fa04782749dab102d45129cdb0b644",
            md5="a97c0affb458a65d8682bf0a48f36e63",
            sha256="c63bf759e5502fc9f4ad863b883423a2d75992aeaebee6a713eb81fe3f714a4b",
            sha1_git="f301245e24299fc112e109874ef38f7340ca0199",
            mime_type="text/plain",
            file_type="ASCII text",
            programming_language="Python",
            is_binary=False,
            is_text=True,
            is_archive=False,
            is_media=False,
            is_source=True,
            is_script=False,
        )
        self.assertEqual(expected, result.file_info)


class PackageResultsTestCase(TestCase):
    def test_rpm(self) -> None:
        with get_from_url(LIBAIO1__0_3_109_1_25__RPM) as rpm_path:
            results = PackageResults.from_rpm(rpm_path)
        self.assertEqual(
            PackageResults(
                api_data_url=None, bug_tracking_url=None, code_view_url=None, copyright=None, datasource_id="rpm_archive",
                declared_license_expression="lgpl-2.1-plus", declared_license_expression_spdx="LGPL-2.1-or-later",
                description=(
                    'Linux-Native Asynchronous I/O Access Library\nThe Linux-native asynchronous I/O facility ("async I/O", or "aio") has\n'
                    'a richer API and capability set than the simple POSIX async I/O\nfacility. This library provides the Linux-native API for async I/O. '
                    'The\nPOSIX async I/O facility requires this library to provide\nkernel-accelerated async I/O capabilities, as do applications that\n'
                    'require the Linux-native async I/O API.'
                ),
                download_url=None, extracted_license_statement="LGPL-2.1+", holder=None,
                homepage_url="http://kernel.org/pub/linux/libs/aio/", keywords=[],
                license_detections=[
                    LicenseDetection(
                        license_expression="lgpl-2.1-plus", license_expression_spdx="LGPL-2.1-or-later",
                        identifier="lgpl_2_1_plus-d0bddf88-13b4-4982-bdf5-ce866a0e8394",
                        matches=[
                            LicenseMatch(
                                score=100.0, start_line=1, end_line=1, matched_length=3, match_coverage=100.0, matcher="1-hash",
                                license_expression="lgpl-2.1-plus", rule_identifier="lgpl-2.1-plus_64.RULE", rule_relevance=100,
                                rule_url="https://github.com/nexB/scancode-toolkit/tree/develop/src/licensedcode/data/rules/lgpl-2.1-plus_64.RULE",
                                matched_text="LGPL-2.1+",
                                license_expression_spdx="LGPL-2.1-or-later", from_file=None,
                            ),
                        ],
                    ),
                ],
                md5=None, name="libaio1", namespace=None, notice_text=None, other_license_detections=[], other_license_expression=None,
                other_license_expression_spdx=None,
                parties=[
                    Party(type=None, role="distributor", name="SUSE Linux Enterprise 15", email=None, url=None),
                    Party(type=None, role="vendor", name="SUSE LLC <https://www.suse.com/>", email=None, url=None),
                ],
                primary_language=None, purl="pkg:rpm/libaio1@0.3.109-1.25", qualifiers={}, release_date=None,
                repository_download_url=None, repository_homepage_url=None, sha1=None, sha256=None, sha512=None, size=None,
                source_packages=["pkg:rpm/libaio@0.3.109-1.25?arch=src"], subpath=None, type="rpm", vcs_url=None, version="0.3.109-1.25",
            ),
            results,
        )


class CleanupTestCase(TestCase):
    def test_cleanup(self) -> None:
        directory_string = mkdtemp()
        directory = Path(directory_string)
        self.assertTrue(directory.is_dir())

        # 1) Existing directory.
        scancode_tools.cleanup(directory)
        self.assertFalse(directory.is_dir())

        # 2) Missing directory.
        scancode_tools.cleanup(directory)
        self.assertFalse(directory.is_dir())



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

from __future__ import annotations

import contextlib
import datetime
from io import BytesIO
from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Any
from unittest import TestCase, mock

import requests

from license_tools.utils import download_utils
from license_tools.utils.download_utils import ChecksumError, Download, DownloadError
from license_tools.utils.path_utils import get_files_from_directory


class DownloadTestCase(TestCase):
    def test_verify_checksum(self) -> None:
        # No checksum.
        Download(url="http://localhost", filename="dummy").verify_checksum(b"Hello World!\n")

        # Correct sha256 checksum.
        Download(
            url="http://localhost", filename="dummy", sha256="03ba204e50d126e4674c005e04d82e84c21366780af1f43bd54a37816b6ab340",
        ).verify_checksum(b"Hello World!\n")

        # Incorrect sha256 checksum.
        with self.assertRaisesRegex(
                expected_exception=ChecksumError,
                expected_regex=r"^Checksum mismatch: Got 03ba204e50d126e4674c005e04d82e84c21366780af1f43bd54a37816b6ab340, expected INVALID!$",
        ):
            Download(url="http://localhost", filename="dummy", sha256="INVALID").verify_checksum(b"Hello World!\n")


class GetSessionTestCase(TestCase):
    def test_get_session(self) -> None:
        session = download_utils.get_session()
        self.assertIsInstance(session, requests.Session)
        self.assertIn("https://github.com/stefan6419846/license_tools version ", session.headers["User-Agent"])


class DownloadFileTestCase(TestCase):
    def test_not_okay(self) -> None:
        session = download_utils.get_session()
        response = requests.Response()
        response.status_code = 404
        file_object = BytesIO()
        with mock.patch.object(download_utils, "get_session", return_value=session), \
                mock.patch.object(session, "get", return_value=response):
            with self.assertRaisesRegex(
                    expected_exception=DownloadError,
                    expected_regex=r"^Download not okay\? http://localhost <Response \[404\]>$",
            ):
                download_utils.download_file(
                    url="http://localhost",
                    file_object=file_object,
                )
        self.assertEqual(b"", file_object.getvalue())

    def test_valid(self) -> None:
        session = download_utils.get_session()
        response = requests.Response()
        response.status_code = 200
        response._content = b"Hello World!\n"
        file_object = BytesIO()

        with mock.patch.object(download_utils, "get_session", return_value=session), \
                mock.patch.object(session, "get", return_value=response):
            download_utils.download_file(
                url="http://localhost",
                file_object=file_object,
            )
        self.assertEqual(b"Hello World!\n", file_object.getvalue())


class DownloadFileToDirectoryTestCase(TestCase):
    def test_reuse_session(self) -> None:
        session = download_utils.get_session()
        with mock.patch.object(download_utils, "get_session") as session_mock:
            with contextlib.suppress(Exception):
                download_utils.download_file_to_directory(
                    download=Download(url="http://localhost", filename="dummy"),
                    directory=Path("/dummy"),
                    session=session,
                )
        session_mock.assert_not_called()

        with mock.patch.object(download_utils, "get_session") as session_mock:
            with contextlib.suppress(Exception):
                download_utils.download_file_to_directory(
                    download=Download(url="http://localhost", filename="dummy"),
                    directory=Path("/dummy"),
                )
        session_mock.assert_called_once_with()

    def test_not_okay(self) -> None:
        session = download_utils.get_session()
        response = requests.Response()
        response.status_code = 404
        with mock.patch.object(session, "get", return_value=response):
            with self.assertRaisesRegex(
                    expected_exception=DownloadError,
                    expected_regex=r"^Download not okay\? http://localhost <Response \[404\]>$",
            ):
                download_utils.download_file_to_directory(
                    download=Download(url="http://localhost", filename="dummy"),
                    directory=Path("/dummy"),
                    session=session,
                )

    def test_hash_mismatch(self) -> None:
        session = download_utils.get_session()
        response = requests.Response()
        response.status_code = 200
        response._content = b"Hello World!\n"

        with mock.patch.object(session, "get", return_value=response):
            with self.assertRaisesRegex(
                    expected_exception=ChecksumError,
                    expected_regex=r"^Checksum mismatch: Got 03ba204e50d126e4674c005e04d82e84c21366780af1f43bd54a37816b6ab340, expected INVALID!$",
            ):
                download_utils.download_file_to_directory(
                    download=Download(url="http://localhost", filename="dummy", sha256="INVALID"),
                    directory=Path("/dummy"),
                    session=session,
                )

    def test_valid(self) -> None:
        session = download_utils.get_session()
        response = requests.Response()
        response.status_code = 200
        response._content = b"Hello World!\n"

        with mock.patch.object(session, "get", return_value=response), TemporaryDirectory() as directory:
            download_utils.download_file_to_directory(
                download=Download(url="http://localhost", filename="test.txt", sha256="03ba204e50d126e4674c005e04d82e84c21366780af1f43bd54a37816b6ab340"),
                directory=Path(directory),
                session=session,
            )
            self.assertEqual(
                b"Hello World!\n",
                Path(directory, "test.txt").read_bytes(),
            )


class DownloadOneFilePerSecondTestCase(TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.timestamps: list[datetime.datetime] = []
        self.downloads = [
            Download(url="http://localhost/file1", filename="file1.txt", sha256="03ba204e50d126e4674c005e04d82e84c21366780af1f43bd54a37816b6ab340"),
            Download(url="http://localhost/file2", filename="file2.txt", sha256="03ba204e50d126e4674c005e04d82e84c21366780af1f43bd54a37816b6ab340"),
            Download(url="http://localhost/file3", filename="file3.txt", sha256="03ba204e50d126e4674c005e04d82e84c21366780af1f43bd54a37816b6ab340"),
        ]

        self.session = download_utils.get_session()
        self._session_patcher = mock.patch.object(download_utils, "get_session", return_value=self.session)
        self._session_patcher.start()
        self.addCleanup(self._session_patcher.stop)

    def test_error(self) -> None:
        # Set wrong hash.
        self.downloads[1].sha256 = "INVALID"

        def get(url: str, *args: Any, **kwargs: Any) -> requests.Response:
            response = requests.Response()
            response.status_code = 200
            response._content = b"Hello World!\n"
            return response

        with mock.patch.object(self.session, "get", side_effect=get), TemporaryDirectory() as directory:
            with self.assertRaisesRegex(
                    expected_exception=ChecksumError,
                    expected_regex=r"^Checksum mismatch: Got 03ba204e50d126e4674c005e04d82e84c21366780af1f43bd54a37816b6ab340, expected INVALID!$",
            ):
                download_utils.download_one_file_per_second(downloads=self.downloads, directory=Path(directory))
            actual = [x[1] for x in get_files_from_directory(directory)]
            self.assertEqual(["file1.txt"], actual)
            self.assertEqual(
                b"Hello World!\n",
                Path(directory, "file1.txt").read_bytes(),
            )

    def test_delays(self) -> None:
        def get(url: str, *args: Any, **kwargs: Any) -> requests.Response:
            self.timestamps.append(datetime.datetime.now())
            response = requests.Response()
            response.status_code = 200
            response._content = b"Hello World!\n"
            return response

        with mock.patch.object(self.session, "get", side_effect=get), TemporaryDirectory() as directory:
            download_utils.download_one_file_per_second(downloads=self.downloads, directory=Path(directory))
            actual = [x[1] for x in get_files_from_directory(directory)]
            self.assertEqual(["file1.txt", "file2.txt", "file3.txt"], actual)
            for name in actual:
                self.assertEqual(
                    b"Hello World!\n",
                    Path(directory, name).read_bytes(),
                    name,
                )

        deltas: list[datetime.timedelta] = [y - x for x, y in zip(self.timestamps[:-1], self.timestamps[1:])]
        for delta in deltas:
            self.assertGreaterEqual(delta.total_seconds(), 1)



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

from __future__ import annotations

import shutil
import subprocess
from pathlib import Path
from tempfile import NamedTemporaryFile
from typing import cast
from unittest import TestCase

from license_tools.tools import translation_tools
from tests import get_file, get_from_url
from tests.data import DJANGO__5076BB4__DJANGO_MO, DJANGO__5076BB4__DJANGO_PO, LICENSE_PATH, SETUP_PATH


class IsCompiledGettextFileTestCase(TestCase):
    def test_is_compiled_gettext_file(self) -> None:
        self.assertFalse(translation_tools.is_compiled_gettext_file(SETUP_PATH))
        self.assertFalse(translation_tools.is_compiled_gettext_file(LICENSE_PATH))

        with get_file("Carlito-Regular.ttf") as path:
            self.assertFalse(translation_tools.is_compiled_gettext_file(path))

        with get_file("croissant.jpg") as path:
            self.assertFalse(translation_tools.is_compiled_gettext_file(path))

        with get_from_url(DJANGO__5076BB4__DJANGO_MO) as path:
            self.assertTrue(translation_tools.is_compiled_gettext_file(path))

        with get_from_url(DJANGO__5076BB4__DJANGO_PO) as path:
            self.assertFalse(translation_tools.is_compiled_gettext_file(path))


class CheckCompiledGettextMetadataTestCase(TestCase):
    maxDiff = None

    def test_python(self) -> None:
        self.assertIsNone(translation_tools.check_compiled_gettext_metadata(SETUP_PATH))

    def test_font(self) -> None:
        with get_file("Carlito-Regular.ttf") as path:
            self.assertIsNone(translation_tools.check_compiled_gettext_metadata(path))

    def test_django_mo(self) -> None:
        with get_from_url(DJANGO__5076BB4__DJANGO_MO) as path:
            result = translation_tools.check_compiled_gettext_metadata(path)

        self.assertIsNotNone(result)

        with get_file("django_mo.po") as path:
            self.assertEqual(path.read_text(), result)

    def test_untranslated_mo_file(self) -> None:
        with get_file("untranslated.po") as po_path:
            expected = po_path.read_text(encoding="UTF-8").splitlines(keepends=False)
            expected = expected[:-3]
            expected = list(filter(lambda x: "POT-Creation-Date" not in x, expected))
            compiled = subprocess.run(
                [
                    cast(str, shutil.which("msgfmt")),
                    po_path,
                    "--output-file", "-",
                ],
                capture_output=True,
                check=True,
            )
        with NamedTemporaryFile(suffix=".mo") as mo_file:
            mo_file.write(compiled.stdout)
            mo_file.seek(0)
            result = translation_tools.check_compiled_gettext_metadata(Path(mo_file.name))
        self.assertIsNotNone(result)
        self.assertListEqual(expected, cast(str, result).splitlines(keepends=False))



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

"""
Tools related to images.
"""

from __future__ import annotations

import logging
import shutil
import subprocess
from pathlib import Path

from license_tools.utils.path_utils import get_mime_type

logger = logging.getLogger(__name__)
del logging


def is_image(path: Path) -> bool:
    """
    Check if the given file is an image file.

    :param path: The file to check.
    :return: True if the file is an image, False otherwise.
    """
    mime_type = get_mime_type(path).lower()
    return mime_type.startswith("image")


def check_image_metadata(path: Path) -> str | None:
    """
    Check the metadata of the given image.

    :param path: The file path to analyze.
    :return: The analysis results if the path points to an image file, `None` otherwise.
    """
    if not is_image(path):
        return None
    exiftool = shutil.which("exiftool")
    if not exiftool:
        raise ValueError("exiftool not found!")

    # The key names might be retrieved by using the `-json` option.
    output = subprocess.check_output(
        [
            exiftool,
            "-duplicates",
            "-groupnames",
            "-charset", "UTF-8",
            "-exclude", "File:Directory",
            "-exclude", "File:FileAccessDate",
            "-exclude", "File:FileInodeChangeDate",
            path,
        ],
        stderr=subprocess.PIPE,
    )
    return output.decode("UTF-8")



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

"""
Archive handling.
"""

from __future__ import annotations

import os
import shutil
from pathlib import Path
from typing import cast

import extractcode  # type: ignore[import-untyped]
from extractcode import all_kinds
from extractcode.api import extract_archive as _extract_archive  # type: ignore[import-untyped]
from extractcode.api import extract_archives as _extract_archives
from extractcode.archive import should_extract as _should_extract  # type: ignore[import-untyped]

# Mitigate https://github.com/aboutcode-org/extractcode/issues/65
# This is a particularly ugly workaround and I would indeed prefer an upstream
# solution, but this would most likely be more complex and require more
# research. In most cases, this should indeed be irrelevant for CLI users,
# but library users might see strange side effects on log statements etc.
# without the fix (at least when not already using UTC as the timezone anyway).
# As far as I could test it, this does not seem to have any relevant
# downsides for license retrieval.
if os.getenv("LICENSE_TOOLS_DISABLE_TZ_WORKAROUND", "").lower() != "true":  # pragma: no cover
    extractcode.libarchive2.set_env_with_tz = lambda: None


def extract(archive_path: Path, target_directory: Path, recurse: bool = False) -> None:
    """
    Extract the given archive recursively.

    :param archive_path: The archive to unpack.
    :param target_directory: The target directory to use.
    :param recurse: Whether to use a recursive approach.
    """
    if recurse:
        targets = set()
        for event in _extract_archives(location=archive_path, all_formats=True, recurse=True):
            if event.done:
                shutil.copytree(src=Path(event.target), dst=target_directory, dirs_exist_ok=True)
                targets.add(Path(event.target))
        for target in targets:
            if target.exists():
                shutil.rmtree(target)
    else:
        for _event in _extract_archive(location=archive_path, target=target_directory):
            pass


def can_extract(archive_path: Path) -> bool:
    """
    Check if the given archive can be extracted.

    :param archive_path: The path to check for.
    :return: The check result.
    """
    return cast(
        bool,
        _should_extract(
            location=archive_path,
            kinds=all_kinds,
        ),
    )
